__author__ = 'aarongary'
import json
from nicecxModel.cx import CX_CONSTANTS

class MetaDataElement(object):
    def __init__(self, elementCount=None, idCounter=None, properties=None, version=None, consistencyGroup=1, lastUpdate=None, name=None, json_obj=None):
        print consistencyGroup
        if json_obj is not None:
            self.element_count = json_obj.get(CX_CONSTANTS.ELEMENT_COUNT)
            self.properties = json_obj.get(CX_CONSTANTS.PROPERTIES)
            self.version = json_obj.get(CX_CONSTANTS.VERSION)
            self.consistencyGroup = json_obj.get(CX_CONSTANTS.CONSISTENCY_GROUP)
            self.name = json_obj.get(CX_CONSTANTS.METADATA_NAME)
            self.id_counter = json_obj.get(CX_CONSTANTS.ID_COUNTER)
            self.last_update = json_obj.get(CX_CONSTANTS.LAST_UPDATE)
        else:
            self.element_count = elementCount
            self.properties = properties
            self.version = version
            self.consistencyGroup = consistencyGroup
            self.name = name
            self.id_counter = idCounter
            self.last_update = lastUpdate

    def getConsistencyGroup(self):
        return self.consistencyGroup

    def getElementCount(self):
        return self.elementCount

    def getIdCounter(self):
        return self.id_counter

    def getLastUpdate(self):
        return self.last_update

    def getName(self):
        return self.name

    def getVersion(self):
        return self.version

    def getProperties(self):
        return self.properties

    def setConsistencyGroup(self, cg):
        self.consistencyGroup = cg

    def incrementConsistencyGroup(self):
        raise Exception('metadata should not be incremented')
        if self.consistencyGroup:
            if type(self.consistencyGroup) is int:
                self.consistencyGroup += 1
            else:
                self.consistencyGroup = int(self.consistencyGroup) + 1
        else:
            self.consistencyGroup = 0

    def setElementCount(self, ec):
        self.element_count = ec

    def setIdCounter(self, ic):
        self.id_counter = ic

    def setLastUpdate(self, lu):
        self.last_update = lu

    def setName(self, n):
        self.name = n

    def setVersion(self, v):
        self.version = v

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        node_dict = {}

        if self.element_count:
            node_dict[CX_CONSTANTS.ELEMENT_COUNT] = self.element_count

        if self.properties:
            node_dict[CX_CONSTANTS.PROPERTIES] = self.properties

        if self.version:
            node_dict[CX_CONSTANTS.VERSION] = self.version

        #if self.consistencyGroup is not None:
        node_dict[CX_CONSTANTS.CONSISTENCY_GROUP] = self.consistencyGroup

        if self.name:
            node_dict[CX_CONSTANTS.METADATA_NAME] = self.name

        if self.id_counter:
            node_dict[CX_CONSTANTS.ID_COUNTER] = self.id_counter

        if self.last_update:
            node_dict[CX_CONSTANTS.LAST_UPDATE] = self.last_update

        return node_dict



__author__ = 'aarongary'

import json
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx import CX_CONSTANTS

class AttributeCommon(object):
    def __init__(self, subnetwork=None, property_of=None, name=None, values=None, type=None, json_obj=None):
        if json_obj is not None:
            data_type = ATTRIBUTE_DATA_TYPE.convert_to_data_type(json_obj.get(CX_CONSTANTS.VALUE))
            self._property_of = json_obj.get(CX_CONSTANTS.PROPERTY_OF)
            self._subnetwork = json_obj.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK)
            self._name = json_obj.get(CX_CONSTANTS.NAME)
            self._values = json_obj.get(CX_CONSTANTS.VALUE)
            self._data_type = data_type
        else:
            self._property_of = property_of
            self._subnetwork = subnetwork
            self._name = name
            self._values = values
            self._data_type = type

        self.ASPECT_NAME = None

    def getPropertyOf(self):
        return self._property_of

    def setPropertyOf(self, id):
        self._property_of = id

    def getSubnetwork(self):
        return self._subnetwork

    def setSubnetwork(self, subnetwork):
        self._subnetwork = subnetwork

    def getName(self):
        return self._name

    def setName(self, name):
        self.name = name

    def getValues(self):
        return self._values

    def setValues(self, values):
        self._values = values

    def getDataType(self):
        return self._data_type

    def setDataType(self, data_type):
        self._data_type = data_type

    def getValueAsJsonString(self):
        return json.dumps(self._values)

    def isSingleValue(self):
        return ATTRIBUTE_DATA_TYPE.isSingleValueType(self._data_type)

    def getAspectName(self):
        return self.ASPECT_NAME

    def setAspectName(self, aspect_name):
        self.ASPECT_NAME = aspect_name

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        return_dict = {}

        if self._property_of is not None:
            return_dict[CX_CONSTANTS.PROPERTY_OF] = self._property_of

        if self._name is not None:
            return_dict[CX_CONSTANTS.NAME] = self._name

        if self._data_type:
            return_dict[CX_CONSTANTS.DATA_TYPE] = self._data_type.value

        if self._subnetwork:
            return_dict[CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK] = self._subnetwork

        if self.isSingleValue():
            return_dict[CX_CONSTANTS.VALUE] = self.getValues()
        else:
            return_dict[CX_CONSTANTS.VALUE] = self._values

        return return_dict



from ndex2.networkn import NdexGraph
import sys


def test_types():
    G = NdexGraph()
    n = G.add_new_node('Node with Types')
    n1 = G.add_new_node('A')
    n2 = G.add_new_node('B')
    G.add_edge_between(n, n1)
    G.add_edge_between(n, n2)
    G.set_name('Test Types')

    G.node[n]['string'] = 'mystring'
    G.node[n]['bool'] = True
    G.node[n]['int'] = 5
    G.node[n]['double'] = 2.5

    #  Python3 doesn't support Long (you cannot have a = 5L);
    #  If we need an integer being a long in Python 2 and still be compatible with Python 3,
    #  we need to define up a long variable to be the same as the int class under Python 3,
    #  and it can then be used explicitly to make sure the integer is a long.
    #
    #  it is taken from http: //python3porting.com/noconv.html
    if sys.version_info.major == 3:
        long = int

    # long(5) will be 5L in Python 2 and just int 5 (which is long) in Python 3
    G.node[n]['long'] = long(5)


    G.node[n]['string_list'] = ['mystring','myotherstring']
    G.node[n]['bool_list'] = [False, True]
    G.node[n]['int_list'] = [5, -20]
    G.node[n]['double_list'] = [2.5, 3.7]

    # long(5), long(75) will be 5L, 75L in Python 2 and just int 5, 75 (which is long) in Python 3
    G.node[n]['long_list'] = [long(5), long(75)]

    G.write_to('temp_test_type.cx')

    # G.upload_to('http://test.ndexbio.org', 'scratch', 'scratch')

def test_metadata():
    G = NdexGraph(server="http://dev.ndexbio.org", uuid="317332f7-ade8-11e6-913c-06832d634f41")
    print(G.metadata_original)

if __name__ == "__main__":
    test_types()
    test_metadata()


__author__ = 'aarongary'

import ndex2
import ijson
import sys
import pysolr
import requests
from pysolr import SolrError
from ndex2 import solr_url
from nicecxModel.NiceCXNetwork import NiceCXNetwork
from nicecxModel.cx.aspects.NodesElement import NodesElement
from nicecxModel.cx.aspects.EdgesElement import EdgesElement
from nicecxModel.cx.aspects.NetworkAttributesElement import NetworkAttributesElement
from nicecxModel.cx.aspects.NodeAttributesElement import NodeAttributesElement
from nicecxModel.cx.aspects.EdgeAttributesElement import EdgeAttributesElement
from nicecxModel.cx.aspects.CitationElement import CitationElement
from nicecxModel.metadata.MetaDataElement import MetaDataElement
from nicecxModel.cx.aspects.SupportElement import SupportElement
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx.aspects.SimpleNode import SimpleNode
from nicecxModel.cx.aspects.AspectElement import AspectElement
from nicecxModel.cx import CX_CONSTANTS
from nicecxModel.cx import known_aspects, known_aspects_min
import time

if sys.version_info.major == 3:
    from urllib.request import urlopen
else:
    from urllib import urlopen


class NetworkQuery():
    def __init__(self):
        self.query = ''

    def query_network_fast(self, uuid, search_string, max_edges):
        niceCx = NiceCXNetwork()
        #uuid = '7246d8cf-c644-11e6-b48c-0660b7976219'
        search_terms_dict = {k:1 for k in search_string.split(',')}
        edge_keepers = set([])
        node_keepers = set([])

        solr = pysolr.Solr(solr_url + uuid + '/', timeout=10)

        try:
            results = solr.search(search_string, rows=10000)
            #search_terms_array = [int(n['id']) for n in results.docs]
            search_terms_array = {int(n['id']):1 for n in results.docs}
            search_terms_set = set([int(n['id']) for n in results.docs])
            if(not search_terms_array):
                return {'message': 'No nodes found'}

            print('starting nodes 1')
            #===================
            # METADATA
            #===================
            available_aspects = []
            for ae in (o for o in self.streamAspect(uuid, 'metaData')):
                available_aspects.append(ae.get(CX_CONSTANTS.METADATA_NAME))
                mde = MetaDataElement(json_obj=ae)
                niceCx.addMetadata(mde)

            opaque_aspects = set(available_aspects).difference(known_aspects_min)

            print(opaque_aspects)

            #===================
            # EDGES
            #===================
            edge_count = 0
            added_edges = 0
            edges = []
            edge_found = False
            edge_id = -1
            edge_s = None
            edge_t = None
            edge_i = None
            e_count = 0

            start_time = time.time()
            parser = self.streamAspectRaw(uuid, 'edges')

            for prefix, event, value in parser:
                # process event, prefix and value

                if(event == 'end_map'):
                    e_count += 1
                    if e_count % 5000 == 0:
                        print(e_count)
                    if edge_found:
                        edge_keepers.add(edge_id)
                        node_keepers.update([edge_s, edge_t])
                        edges.append(EdgesElement(id=edge_id, edge_source=edge_s, edge_target=edge_t, edge_interaction=edge_i))

                    edge_id = -1
                    edge_s = None
                    edge_t = None
                    edge_i = None
                    edge_found = False

                if (prefix) == ('item.s'):
                    if value in search_terms_set:
                        edge_found = True
                    edge_s = value

                elif (prefix) == ('item.t'):
                    if value in search_terms_set:
                        edge_found = True
                    edge_t = value

                elif (prefix) == ('item.i'):
                    if edge_found:
                        edge_i = value

                elif (prefix) == ('item.@id'):
                    edge_id = value

            print('Response time (Edge search): ' + str(time.time() - start_time))
            print(node_keepers)
            print(edge_keepers)

            '''
            if 'edges' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'edges')):
                    if ae.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID) in search_terms_set or ae.get(CX_CONSTANTS.EDGE_TARGET_NODE_ID) in search_terms_set:
                        edge_keepers.add(ae.get(CX_CONSTANTS.ID))
                        node_keepers.update([ae.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID), ae.get(CX_CONSTANTS.EDGE_TARGET_NODE_ID)])

                    if edge_count % 5000 == 0:
                        print(edge_count)

                    if added_edges > max_edges:
                        raise StopIteration('Max edges reached')

                    edge_count += 1
            else:
                raise Exception('Network does not contain any nodes.  Cannot query')
            '''

        except SolrError as se:
            if('404' in se.message):
                ndex2.get_logger('SOLR').warning('Network not found ' + self.uuid + ' on ' + solr_url + ' server.')
                raise Exception("Network not found (SOLR)")
            else:
                ndex2.get_logger('SOLR').warning('Network error ' + self.uuid + ' on ' + solr_url + ' server. ' + se.message)
                raise Exception(se.message)
        except StopIteration as si:
                ndex2.get_logger('QUERY').warning("Found more than max edges.  Raising exception")
                raise StopIteration(si.message)


    def query_network(self, uuid, search_string, max_edges):
        myConst = CX_CONSTANTS

        niceCx = NiceCXNetwork()
        #uuid = '7246d8cf-c644-11e6-b48c-0660b7976219'
        search_terms_dict = {k:1 for k in search_string.split(',')}

        solr = pysolr.Solr(solr_url + uuid + '/', timeout=10)

        try:
            results = solr.search(search_string, rows=10000)
            #search_terms_array = [int(n['id']) for n in results.docs]
            search_terms_array = {int(n['id']):1 for n in results.docs}
            if(not search_terms_array):
                return {'message': 'No nodes found'}

            print('starting nodes 1')
            #===================
            # METADATA
            #===================
            available_aspects = []
            for ae in (o for o in self.streamAspect(uuid, 'metaData')):
                available_aspects.append(ae.get(CX_CONSTANTS.METADATA_NAME))
                mde = MetaDataElement(json_obj=ae)
                niceCx.addMetadata(mde)

            #available_aspects = ['edges', 'nodes'] # TODO - remove this
            opaque_aspects = set(available_aspects).difference(known_aspects_min)

            print(opaque_aspects)

            #===================
            # NODES
            #===================
            if 'nodes' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'nodes')):
                    if search_terms_array.get(ae.get(CX_CONSTANTS.ID)):
                        add_this_node = NodesElement(json_obj=ae)
                        niceCx.addNode(add_this_node)
            else:
                raise Exception('Network does not contain any nodes.  Cannot query')

            print('starting edges 1')
            #===================
            # EDGES
            #===================
            edge_count = 0
            added_edges = 0
            start_time = time.time()
            if 'edges' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'edges')):
                    if niceCx.nodes.get(ae.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK)) is not None or niceCx.nodes.get(ae.get(CX_CONSTANTS.EDGE_TARGET_NODE_ID)) is not None:
                        add_this_edge = EdgesElement(json_obj=ae)
                        niceCx.addEdge(add_this_edge)
                        added_edges += 1
                    if edge_count % 5000 == 0:
                        print(edge_count)

                    #if edge_count > 30000:
                    #    break

                    if added_edges > max_edges:
                        raise StopIteration('Max edges reached')
                    edge_count += 1
            else:
                raise Exception('Network does not contain any nodes.  Cannot query')

            print('Response time (Edge search): ' + str(time.time() - start_time))
            print('starting nodes 2')
            #===================
            # NODES
            #===================
            for ae in (o for o in self.streamAspect(uuid, 'nodes')):
                if niceCx.getMissingNodes().get(ae.get(CX_CONSTANTS.ID)):
                    add_this_node = NodesElement(json_obj=ae)
                    niceCx.addNode(add_this_node)

            #====================
            # NETWORK ATTRIBUTES
            #====================
            if 'networkAttributes' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'networkAttributes')):
                    add_this_network_attribute = NetworkAttributesElement(json_obj=ae)
                    niceCx.addNetworkAttribute(add_this_network_attribute)

            #===================
            # NODE ATTRIBUTES
            #===================
            if 'nodeAttributes' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'nodeAttributes')):
                    if niceCx.nodes.get(ae.get(CX_CONSTANTS.PROPERTY_OF)):
                        add_this_node_att = NodeAttributesElement(json_obj=ae)
                        niceCx.addNodeAttribute(add_this_node_att)

            #===================
            # EDGE ATTRIBUTES
            #===================
            if 'edgeAttributes' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'edgeAttributes')):
                    if niceCx.edges.get(ae.get(CX_CONSTANTS.PROPERTY_OF)):
                        add_this_edge_att = EdgeAttributesElement(json_obj=ae)
                        niceCx.addEdgeAttribute(add_this_edge_att)

            #===================
            # NODE CITATIONS
            #===================
            if 'nodeCitations' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'nodeCitations')):
                    for e_po in ae.get(CX_CONSTANTS.PROPERTY_OF):
                        if niceCx.getNodes().get(e_po) is not None:
                            niceCx.addNodeCitationsFromCX(ae)

            #===================
            # EDGE CITATIONS
            #===================
            ec_count = 0
            if 'edgeCitations' in available_aspects:
                for ae in (o for o in self.streamAspect(uuid, 'edgeCitations')):
                    for e_po in ae.get(CX_CONSTANTS.PROPERTY_OF):
                        if niceCx.getEdges().get(e_po) is not None:
                            niceCx.addEdgeCitationsFromCX(ae)
                    ec_count += 1
                    if ec_count % 500 == 0:
                        print(ec_count)

            #===================
            # CITATIONS
            #===================
            if 'citations' in available_aspects:
                #======================================================
                # FILTER CITATIONS IF THERE ARE EDGE OR NODE CITATIONS
                # OTHERWISE ADD THEM ALL (NO-FILTER) -- TODO
                #======================================================
                for ae in (o for o in self.streamAspect(uuid, 'citations')):
                    add_this_citation = CitationElement(json_obj=ae)
                    niceCx.addCitation(add_this_citation)

            #===================
            # OPAQUE ASPECTS
            #===================
            for oa in opaque_aspects:
                objects = self.streamAspect(uuid, oa)
                obj_items = (o for o in objects)
                for oa_item in obj_items:
                    aspect_element = AspectElement(oa_item, oa)
                    niceCx.addOpapqueAspect(aspect_element)

        except SolrError as se:
            if('404' in se.message):
                ndex2.get_logger('SOLR').warning('Network not found ' + self.uuid + ' on ' + solr_url + ' server.')
                raise Exception("Network not found (SOLR)")
            else:
                ndex2.get_logger('SOLR').warning('Network error ' + self.uuid + ' on ' + solr_url + ' server. ' + se.message)
                raise Exception(se.message)
        except StopIteration as si:
                ndex2.get_logger('QUERY').warning("Found more than max edges.  Raising exception")
                raise StopIteration(si.message)


        #nice_cx_json = niceCx.to_json()

        return niceCx

    def streamAspect(self, uuid, aspect_name):
        if aspect_name == 'metaData':
            print('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect')
            md_response = requests.get('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect')
            json_respone = md_response.json()
            return json_respone.get('metaData')
        else:
            return ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/' + aspect_name), 'item')

    def streamAspectRaw(self, uuid, aspect_name):
        return ijson.parse(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/' + aspect_name))



from __future__ import absolute_import

import unittest
from os import path
import ndex2.client as nc
import uuid
import time
import json


## The following 2 test users have to be created on the test server before running the test suite.

TESTSERVER= "http://dev2.ndexbio.org"
HERE = path.abspath(path.dirname(__file__))

testUser1= 'scratch' #'pytest'
testUser2= 'scratch3' #'pytest2'
testUserpasswd = 'scratch' # 'pyunittest'

class NdexClientTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls._ndex = nc.Ndex2(host=TESTSERVER, username=testUser1, password=testUserpasswd, debug=True)
        with open(path.join(HERE, 'tiny_network.cx'),'r') as cx_file:
            cls._nrc = cls._ndex.save_cx_stream_as_new_network(cx_file)
            networkId = uuid.UUID('{'+cls._nrc[-36:] + '}')
            cls._networkId = str(networkId)
            time.sleep(2)

    @classmethod
    def tearDownClass(cls):
        cls._ndex.delete_network(cls._networkId)
        print("Network " + cls._networkId + " deleted from " + cls._ndex.username +" account " + cls._ndex.host)

class NdexClientRASMachineTestCase(NdexClientTestCase):
    @classmethod
    def setUpClass(cls):
        cls._ndex = nc.Ndex2(host=TESTSERVER, username=testUser1, password=testUserpasswd, debug=True)
        with open(path.join(HERE, 'The_RAS_Machine.cx'),'r') as cx_file:
            cls._nrc = cls._ndex.save_cx_stream_as_new_network(cx_file)
            networkId = uuid.UUID('{'+cls._nrc[-36:] + '}')
            cls._networkId = str(networkId)
            time.sleep(2)

class NdexClientTestCase1(unittest.TestCase):

    def testConstructorException(self):
        with self.assertRaises(Exception):
            print("testing ndex client constructor.")
            ndex = nc.Ndex2(host="www.google.com", username="foo", password="bar", debug=True)

    def testConstructor2 (self):
        ndex = nc.Ndex2(host=TESTSERVER,update_status=True,debug=True)
        self.assertTrue(ndex.status.get('properties')['ServerVersion'].startswith("2."))


class NdexClientTestCase2(NdexClientTestCase):


    def testGetNetwork(self):
        summary = self._ndex.get_network_summary(self._networkId)
        self.assertEqual(summary.get(u'externalId'), self._networkId)
        print("get_network_summary() passed.")

    @unittest.skip("group uuid is not valid")
    def testGrantGroupPermission(self):
        ndex2 = nc.Ndex2(TESTSERVER, username=testUser2 , password = testUserpasswd,debug=True)
        with self.assertRaises(Exception) as context:
            ndex2.get_network_summary(self._networkId)
#        print context.exception
        count = 0
        while count < 30 :
            try :
                self._ndex.update_network_group_permission('d7ef9957-de81-11e6-8835-06832d634f41', self._networkId, 'READ')
                count = 60
            except Exception as inst :
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification") :
                    print("retry in 3 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(3)
                else :
                    raise inst

        summary = ndex2.get_network_summary(self._networkId)
        self.assertEqual(summary.get(u'externalId'), self._networkId)
        print("update_network_group_permission() passed.")



class NdexClientTestCase3(NdexClientTestCase):

    def testGrantUserPermission(self):
        ndex2 = nc.Ndex2(TESTSERVER, username=testUser2 , password = testUserpasswd, debug=True)
        with self.assertRaises(Exception) as context:
            ndex2.get_network_summary(self._networkId)
#        print context.exception
        count = 0
        while count < 30 :
            try :
                # User UUID of testUser2 in this test
                self._ndex.update_network_user_permission('0a9d3b58-de82-11e6-8835-06832d634f41', self._networkId, 'READ')
                break
            except Exception as inst :
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification") :
                    print("retry in 3 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(3)
                else :
                    raise inst

        summary = ndex2.get_network_summary(self._networkId)
        self.assertEqual(summary.get(u'externalId'), self._networkId)
        print("update_network_user_permission() passed.")


class NdexClientTestCase3(NdexClientTestCase):

    def testUpdateNetwork(self):
        time.sleep(5)
        summary1 = self._ndex.get_network_summary(self._networkId)
        self.assertEqual(summary1['name'], 'rudi')
        time.sleep(5)
        count = 0
        while count < 30 :
            try :
                with open(path.join(HERE, 'A549-SL-network.cx'), 'r') as cx_file:
                    self._ndex.update_cx_network(cx_file, self._networkId)
                break
            except Exception as inst :
                if inst.response and inst.response.get('content') :
                    d = json.loads(inst.response.content)
                    if d and d.get('errorCode') and d.get('errorCode').startswith("NDEx_Concurrent_Modification") :
                        print("retry in 1 seconds(" + str(count) + ")")
                        count += 1
                        time.sleep(1)
                    else:
                        raise inst
                else :
                    raise inst

      #  time.sleep(1)

        count = 0
        while count < 30:
            try:
                summary1 = self._ndex.get_network_summary(self._networkId)
                if ( summary1.get('isValid') ):
                    break
                else:
                     count+=1
            except Exception as inst:
                if (not inst.response) or (not inst.response.get('content')):
                    raise inst
                d = json.loads(inst.response.content)
                if d and d.get('errorCode') and d.get('errorCode').startswith("NDEx_Concurrent_Modification"):
                    print("retry in 1 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(1)
                else:
                    raise inst

        self.assertEqual(summary1.get(u'name'), "A549-SL-network")
        print("update_cx_network() passed.")


if __name__ == '__main__':
    unittest.main()



__author__ = 'aarongary'

from nicecxModel.cx import CX_CONSTANTS
import json

class EdgesElement(object):
    def __init__(self, id=None, edge_source=None, edge_target=None, edge_interaction=None, json_obj=None):
        self.ASPECT_NAME = 'edges'

        self._source = None
        self._target = None
        self._interaction = None
        self._id = -1

        if id is not None:
            self._id = id

        if edge_source is not None:
            self._source = edge_source

        if edge_target is not None:
            self._target = edge_target

        if edge_interaction is not None:
            self._interaction = edge_interaction

        if json_obj is not None:
            if type(json_obj) is dict:
                self._source = json_obj.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK)
                self._target = json_obj.get(CX_CONSTANTS.EDGE_TARGET_NODE_ID)
                self._interaction = json_obj.get(CX_CONSTANTS.EDGE_INTERACTION)
                if json_obj.get(CX_CONSTANTS.ID) is not None:
                    self._id = json_obj.get(CX_CONSTANTS.ID)
            else:
                raise Exception('EdgesElement json input provided was not of type json object.')

    def getAspectName(self):
        return self.ASPECT_NAME

    def getId(self):
        return self._id

    def setId(self, id):
        self._id = id

    def getSource(self):
        return self._source

    def setSource(self, source):
        self._source = source

    def getTarget(self):
        return self._target

    def setTarget(self, target):
        self._target = target

    def getInteraction(self):
        return self._interaction

    def setInteraction(self, interaction):
        self._interaction = interaction

    def __eq__ (self, other):
        return self == other or self._id == other._id

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        if self._id == -1:
            raise Exception('Edge element does not have a valid ID.  Unable to process this edge')

        node_dict = {
            CX_CONSTANTS.ID: self._id,
        }

        if self._source is not None:
            node_dict[CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK] = self._source

        if self._target is not None:
            node_dict[CX_CONSTANTS.EDGE_TARGET_NODE_ID] = self._target

        if self._interaction is not None:
            node_dict[CX_CONSTANTS.EDGE_INTERACTION] = self._interaction

        return node_dict




__author__ = 'aarongary'

import json
#from AbstractElementAttributesAspectElement import AbstractElementAttributesAspectElement
from . import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx.aspects.AttributeCommon import AttributeCommon

class NodeAttributesElement(AttributeCommon):
    def __init__(self, subnetwork=None, property_of=None, name=None, values=None, type=None, json_obj=None):
        super(NodeAttributesElement, self).__init__(subnetwork=subnetwork, property_of=property_of, name=name, values=values, type=type, json_obj=json_obj)
        self.setAspectName('nodeAttributes')







__author__ = 'aarongary'

from . import ATTRIBUTE_DATA_TYPE

class DatamodelsUtil():

    def removeParenthesis(self, string, type):
        if string is None:
            return None

        substring = string.strip()
        if substring == 'None' or substring == 'null':
            return None

        if substring.startswith('"') and substring.endswith('"'):
            substring = substring[1:-1]

        if type == ATTRIBUTE_DATA_TYPE.STRING and len() < 1:
            raise Exception("illegal format, empty strings not allowed: " + string)

        return substring


    def parseStringToStringList(self, string, type):
        l = []
        if string is None:
            return None

        str = string.strip()
        if str.startswith("[") and str.endswith("]"):
            str_split = str.split()
            for s in str_split:
                if s == 'null':
                   l.append(None)
                else:
                    l.append(s)
        else:
            raise Exception('parsing string to string list: illegal format: expected to begin and end with square brackets, instead got : ' + str)

        return l;




__author__ = 'aarongary'

import json

class NameSpaces():
    def __init__(self):
	    self.serialVersionUID = 1
	    self.ASPECT_NAME = '@context'

    def getAspectName(self):
        return self.ASPECT_NAME

    def compareTo(self, o):
        if o is not None and o.getAspectName() is not None and self.getAspectName() is not None:
            if self.getAspectName() > o.getAspectName():
                return 1
            elif self.getAspectName() < o.getAspectName():
                return -1
            else:
                return 0

	#def write(self, out):
	#	out.writeObject(this);




__author__ = 'aarongary'

import json
import ijson
import requests
import base64
import sys
#from urllib import urlopen
from ndex2.client import Ndex2
from nicecxModel.NiceCXNetwork import NiceCXNetwork
from nicecxModel.cx.aspects.NodesElement import NodesElement
from nicecxModel.cx.aspects.EdgesElement import EdgesElement
from nicecxModel.cx.aspects.NetworkAttributesElement import NetworkAttributesElement
from nicecxModel.cx.aspects.NodeAttributesElement import NodeAttributesElement
from nicecxModel.cx.aspects.EdgeAttributesElement import EdgeAttributesElement
from nicecxModel.cx.aspects.CitationElement import CitationElement
from nicecxModel.cx.aspects.SupportElement import SupportElement
from nicecxModel.cx.aspects.AspectElement import AspectElement
from nicecxModel.metadata.MetaDataElement import MetaDataElement
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx.aspects.SimpleNode import SimpleNode
from nicecxModel.cx import CX_CONSTANTS
from nicecxModel.cx import known_aspects, known_aspects_min

if sys.version_info.major == 3:
    from urllib.request import urlopen, Request, HTTPBasicAuthHandler, HTTPPasswordMgrWithDefaultRealm, \
        build_opener, install_opener, HTTPError, URLError
else:
    from urllib2 import urlopen, Request, HTTPBasicAuthHandler, HTTPPasswordMgrWithDefaultRealm, \
        build_opener, install_opener, HTTPError, URLError


class NiceCXBuilder():
    def __init__(self, cx=None, server=None, username='scratch', password='scratch', uuid=None, networkx_G=None, data=None, **attr):
        self.user_base64 = None
        self.username = None
        self.password = None
        if username and password:
            self.username = username
            self.password = password
            if sys.version_info.major == 3:
                encode_string = '%s:%s' % (username, password)
                byte_string = encode_string.encode()
                self.user_base64 = base64.b64encode(byte_string)#.replace('\n', '')
            else:
                self.user_base64 = base64.encodestring('%s:%s' % (username, password)).replace('\n', '')

    def create_from_server(self, server, username, password, uuid):
        if server and uuid:
            niceCx = NiceCXNetwork()

            #===================
            # METADATA
            #===================
            available_aspects = []
            for ae in (o for o in self.streamAspect(uuid, 'metaData')):
                available_aspects.append(ae.get(CX_CONSTANTS.METADATA_NAME))
                mde = MetaDataElement(json_obj=ae)
                niceCx.addMetadata(mde)

            #available_aspects = ['edges', 'nodes'] # TODO - remove this
            opaque_aspects = set(available_aspects).difference(known_aspects_min)

            print(opaque_aspects)

            #====================
            # NETWORK ATTRIBUTES
            #====================
            objects = self.streamAspect(uuid, 'networkAttributes')
            obj_items = (o for o in objects)
            for network_item in obj_items:
                add_this_network_attribute = NetworkAttributesElement(json_obj=network_item)

                niceCx.addNetworkAttribute(add_this_network_attribute)

            #===================
            # NODES
            #===================
            objects = self.streamAspect(uuid, 'nodes')
            obj_items = (o for o in objects)
            for node_item in obj_items:
                add_this_node = NodesElement(json_obj=node_item)

                niceCx.addNode(add_this_node)

            #===================
            # EDGES
            #===================
            objects = self.streamAspect(uuid, 'edges')
            obj_items = (o for o in objects)
            for edge_item in obj_items:
                add_this_edge = EdgesElement(json_obj=edge_item)

                niceCx.addEdge(add_this_edge)

            #===================
            # NODE ATTRIBUTES
            #===================
            objects = self.streamAspect(uuid, 'nodeAttributes')
            obj_items = (o for o in objects)
            for att in obj_items:
                add_this_node_att = NodeAttributesElement(json_obj=att)

                niceCx.addNodeAttribute(add_this_node_att)

            #===================
            # EDGE ATTRIBUTES
            #===================
            objects = self.streamAspect(uuid, 'edgeAttributes')
            obj_items = (o for o in objects)
            for att in obj_items:
                add_this_edge_att = EdgeAttributesElement(json_obj=att)

                niceCx.addEdgeAttribute(add_this_edge_att)

            #===================
            # CITATIONS
            #===================
            objects = self.streamAspect(uuid, 'citations')
            obj_items = (o for o in objects)
            for cit in obj_items:
                add_this_citation = CitationElement(json_obj=cit)

                niceCx.addCitation(add_this_citation)

            #===================
            # SUPPORTS
            #===================
            objects = self.streamAspect(uuid, 'supports')
            obj_items = (o for o in objects)
            for sup in obj_items:
                add_this_supports = SupportElement(json_obj=sup)

                niceCx.addSupport(add_this_supports)

            #===================
            # NODE CITATIONS
            #===================
            objects = self.streamAspect(uuid, 'nodeCitations')
            obj_items = (o for o in objects)
            for node_cit in obj_items:
                niceCx.addNodeCitationsFromCX(node_cit)

            #===================
            # EDGE CITATIONS
            #===================
            objects = self.streamAspect(uuid, 'edgeCitations')
            obj_items = (o for o in objects)
            for edge_cit in obj_items:
                niceCx.addEdgeCitationsFromCX(edge_cit)

            #===================
            # OPAQUE ASPECTS
            #===================
            for oa in opaque_aspects:
                objects = self.streamAspect(uuid, oa)
                obj_items = (o for o in objects)
                for oa_item in obj_items:
                    aspect_element = AspectElement(oa_item, oa)
                    niceCx.addOpapqueAspect(aspect_element)

            return niceCx
        else:
            raise Exception('Server and uuid not specified')

    def create_from_cx(self, cx):
        niceCx = NiceCXNetwork()
        for aspect in cx:
            if 'status' in aspect :
                if aspect['status'][0]['success']:
                    continue
                else:
                    raise RuntimeError("Error in CX status aspect: " + aspect['status'][0]['error'])
            if "numberVerification" in aspect:
                # new status and numberVerification will be added when the network is output to_cx
                continue
            if 'subNetworks' in aspect:
                for subnetwork in aspect.get('subNetworks'):
                    id = subnetwork.get('@id')
                    if self.subnetwork_id != None:
                        raise ValueError("niceCX does not support collections!")
                    self.subnetwork_id = id
            elif 'cyViews' in aspect:
                for cyViews in aspect.get('cyViews'):
                    id = cyViews.get('@id')
                    if self.view_id != None:
                        raise ValueError("niceCX does not support more than one view!")
                    self.view_id = id
            elif 'metaData' in aspect:
                available_aspects = []
                for ae in (o for o in aspect.get('metaData')):
                    available_aspects.append(ae.get(CX_CONSTANTS.METADATA_NAME))
                    mde = MetaDataElement(json_obj=ae)
                    niceCx.addMetadata(mde)
                opaque_aspects = set(available_aspects).difference(known_aspects)

                continue
            elif 'provenanceHistory' in aspect:
                elements = aspect['provenanceHistory']
                if len(elements) > 0:
                    if len(elements)>1 or self.provenance :
                        raise RuntimeError('profenanceHistory aspect can only have one element.')
                    else :
                        self.provenance = elements[0]
            elif '@context' in aspect :
                elements = aspect['@context']
                if len(elements) > 0:
                    if  len(elements) > 1 or self.namespaces:
                        raise RuntimeError('@context aspect can only have one element')
                    else :
                        self.namespaces = elements[0]
            # TODO elif if it's an aspect we want to keep out we put an elif for that aspect
            else:
                self.unclassified_cx.append(aspect)

            cx = self.unclassified_cx

    def loadAspect(self, aspect_name):
        #with open('Signal1.cx', mode='r') as cx_f:
        with open('network1.cx', mode='r') as cx_f:
            aspect_json = json.loads(cx_f.read())
            for aspect in aspect_json:
                if aspect.get(aspect_name) is not None:
                    return aspect.get(aspect_name)

    def streamAllAspects(self, uuid):
        return ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid), 'item')

    def streamAspect(self, uuid, aspect_name):
        if aspect_name == 'metaData':
            print('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect')
            md_response = requests.get('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect')
            json_respone = md_response.json()
            return json_respone.get('metaData')
        else:
            #password_mgr = HTTPPasswordMgrWithDefaultRealm()

            #top_level_url = 'http://dev2.ndexbio.org'
            #password_mgr.add_password(None, top_level_url, self.username, self.password)
            #handler = HTTPBasicAuthHandler(password_mgr)
            #opener = build_opener(handler)
            #install_opener(opener)





            # Create an OpenerDirector with support for Basic HTTP Authentication...
            #auth_handler = HTTPBasicAuthHandler()
            #auth_handler.add_password(None, 'http://dev2.ndexbio.org', 'scratch', 'scratch')
            #opener = build_opener(auth_handler)
            # ...and install it globally so it can be used with urlopen.
            #install_opener(opener)


            request = Request('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/' + aspect_name)
            base64string = base64.b64encode('%s:%s' % ('scratch', 'scratch'))
            request.add_header("Authorization", "Basic %s" % base64string)
            #result = urllib2.urlopen(request)


            urlopen_result = None
            try:
                urlopen_result = urlopen(request) #'http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/' + aspect_name)
            except HTTPError as e:
                print(e.code)
                return []
            except URLError as e:
                print('Other error')
                print('URL Error %s' % e.message())
                return []

            return ijson.items(urlopen_result, 'item')

    def streamAspectRaw(self, uuid, aspect_name):
        return ijson.parse(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/' + aspect_name))





import json
import pandas as pd
import csv
import networkx as nx
from nicecxModel.NiceCXNetwork import NiceCXNetwork
from nicecxModel.cx.aspects.NodesElement import NodesElement
from nicecxModel.cx.aspects.EdgesElement import EdgesElement

#with open('CTD_genes_pathways.txt', 'rU') as tsvfile:
#with open('CTD_chem_path_enriched.txt', 'rU') as tsvfile:
with open('SIMPLE3.txt', 'rU') as tsvfile:
    #header = [h.strip() for h in tsvfile.next().split('\t')]

    #df = pd.read_csv(tsvfile,delimiter='\t',engine='python',names=header)
    df = pd.read_csv(tsvfile,delimiter='\t',engine='python',header=None)

    #====================================
    # BUILD NICECX FROM PANDAS DATAFRAME
    #====================================
    niceCx = NiceCXNetwork()
    #niceCx.create_from_pandas(df, source_field='GeneSymbol', target_field='PathwayName', source_node_attr=['GeneID'], target_node_attr=['Pathway Source'], edge_attr=[])
    #niceCx.create_from_pandas(df, source_field='ChemicalName', target_field='PathwayName', source_node_attr=['Chemical ID (MeSH)'], target_node_attr=['Pathway Source'], edge_attr=[])
    niceCx.create_from_pandas(df)
    #my_cx_json = niceCx.to_json()
    print 'nx created'
    niceCx.apply_template('dev2.ndexbio.org', 'scratch', 'scratch', '3daff7cd-9a6b-11e7-9743-0660b7976219')
    print 'template applied'

    niceCx.upload_new_network_stream('dev2.ndexbio.org', 'scratch', 'scratch')

    #niceCx.upload_to('dev2.ndexbio.org', 'scratch', 'scratch')
    print 'Done!'




from six import string_types, integer_types
from sys import version_info
PY3 = version_info > (3,)


def number_verification():
    return [{'numberVerification': [{'longNumber': 281474976710655}]}]


def metadata(metadata_dict=None, max_node_id=0, max_edge_id=0):
    if (metadata_dict is not None):
        metadata_list = [{'name': k, 'idCounter': v, 'consistencyGroup': 2} for k, v in metadata_dict.items()]
        return [{'metaData': metadata_list}]
    else:
        return [
            {'metaData': [{'name': 'nodes', 'idCounter': max_node_id}, {'name': 'edges', 'idCounter': max_edge_id}]}]


def subnetworks(G, subnetwork_id, view_id):
    result = [{'subNetworks': [{'@id': subnetwork_id,
                                'nodes': G.nodes(),
                                'edges': [e[2] for e in G.edges_iter(keys=True)]}]}]
    result += [{'cyViews': [{'s': subnetwork_id, '@id': view_id}]}]
    result += [{"networkRelations": [{"p": subnetwork_id,
                                      "c": view_id,
                                      "r": "view",
                                      "name": "%s view" % G.get_name()
                                      },
                                     {"c": subnetwork_id,
                                      "r": "subnetwork",
                                      "name": G.get_name(),
                                      "p": 10000 # this is a bogus number provided just to enable CyNDEx import
                                      }]
                }]
    return result


def nodes(G):
    return [
        {'nodes': [{'@id': n[0], 'n': n[1]['name'], 'r': n[1]['represents']}]} if 'name' in n[1] and 'represents' in n[
            1] else
        {'nodes': [{'@id': n[0], 'n': n[1]['name']}]} if 'name' in n[1] and 'represents' not in n[1] else
        {'nodes': [{'@id': n[0], 'r': n[1]['represents']}]} if 'name' not in n[1] and 'represents' in n[1] else
        {'nodes': [{'@id': n[0]}]}
        for n in G.nodes_iter(data=True)]


def edges(G):
    return [
        {'edges': [{'i': e[3]['interaction'], 'k': e[3]['keep'], 's': e[0], '@id': e[2], 't': e[1]}]}
        if 'interaction' in e[3] and 'keep' in e[3] else
        {'edges': [{'i': e[3]['interaction'], 's': e[0], '@id': e[2], 't': e[1]}]}
        if 'interaction' in e[3] and 'keep' not in e[3] else
        {'edges': [{'s': e[0], '@id': e[2], 't': e[1]}]}
        for e in G.edges_iter(data=True, keys=True)]


def network_attributes(G, has_single_subnetwork):
    elements = []
    for attribute in G.graph:
        value = G.graph[attribute]
        element = {'n': attribute, 'v': value}
        if not isinstance(value, string_types):
            d = domain(value)
            if d == "unknown":
                if isinstance(value, dict):
                    d = "dict"

            element["d"] = d
        if has_single_subnetwork:
            element["s"] = G.subnetwork_id
        elements.append(element)
    return [{'networkAttributes': elements}]

    # return [{'networkAttributes': [
    #     {'n': k, 'v': G.graph[k]} if isinstance(G.graph[k], string_types) else
    #     {'n': k, 'v': cv(G.graph[k]), 'd': domain(G.graph[k])}
    #     for k in G.graph]}]


def node_attributes(G, has_single_subnetwork):
    elements = []
    try:
        for node_id, attributes in G.nodes_iter(data=True):
            for attribute_name in attributes:
                if attribute_name != "name" and attribute_name != "represents":
                    attribute_value = attributes[attribute_name]
                    element = {'po': node_id, 'n': attribute_name, 'v': attribute_value}

                    if not isinstance(attribute_value, string_types):
                        element['d'] = domain(attribute_value)

                    if has_single_subnetwork:
                        element['s'] = G.subnetwork_id

                    elements.append(element)
        if len(elements) == 0:
            return None
        else:
            return [{"nodeAttributes": elements}]

    except Exception as e:
        print(e.message)


    # return [{'nodeAttributes': [
    #     {'po': n[0], 'n': k, 'v': n[1][k]} if isinstance(n[1][k], string_types) else
    #     {'po': n[0], 'n': k, 'v': cv(n[1][k]), 'd': domain(n[1][k])}
    #     for k in n[1] if k != 'name' and k != 'represents']} for n in G.nodes_iter(data=True)
    #         if ('name' in n[1] and 'represents' in n[1] and len(n[1]) > 2) or
    #         ('name' in n[1] and 'represents' not in n[1] and len(n[1]) > 1) or
    #         ('name' not in n[1] and 'represents' in n[1] and len(n[1]) > 1) or
    #         ('name' not in n[1] and 'represents' not in n[1] and len(n[1]) > 0)]


def edge_attributes(G, has_single_subnetwork):
    elements = []
    for source, target, edge_id, attributes in G.edges_iter(data=True, keys=True):
        for attribute_name in attributes:
            if attribute_name == 'interaction':
                continue
            attribute_value = attributes[attribute_name]
            element = {'po': edge_id, 'n': attribute_name, 'v': attribute_value}

            if not isinstance(attribute_value, string_types):
                element['d'] = domain(attribute_value)

            if has_single_subnetwork:
                element['s'] = G.subnetwork_id

            elements.append(element)

    if len(elements) == 0:
        return None
    else:
        return [{"edgeAttributes": elements}]

    # return [{'edgeAttributes': [
    #     {'po': e[2], 'n': k, 'v': e[3][k]} if isinstance(e[3][k], string_types) else
    #     {'po': e[2], 'n': k, 'v': cv(e[3][k]), 'd': domain(e[3][k])}
    #     for k in e[3]]} for e in G.edges_iter(data=True, keys=True) if e[3]]


def cartesian(G, id):
    return [{'cartesianLayout': [
        {'node': n, 'view': id, 'x': float(G.pos[n][0]), 'y': float(G.pos[n][1])}
        for n in G.pos
        ]}]

def cartesian(G):
    return [{'cartesianLayout': [
        {'node': n, 'x': float(G.pos[n][0]), 'y': float(G.pos[n][1])}
        for n in G.pos
        ]}]

def citations(G):
    citations = []
    for citation_id in G.citation_map:
        citation = G.citation_map[citation_id]
        citation["@id"] = citation_id
        citations.append(citation)
    return [{"citations": citations}]


def node_citations(G):
    node_citations = []
    for node_id in G.node_citation_map:
        citations = G.node_citation_map[node_id]
        node_citations.append({"citations": citations, "po": [node_id]})
    return [{"nodeCitations": node_citations}]


def edge_citations(G):
    edge_citations = []
    for edge_id in G.edge_citation_map:
        citations = G.edge_citation_map[edge_id]
        edge_citations.append({"citations": citations, "po": [edge_id]})
    return [{"edgeCitations": edge_citations}]


def supports(G):
    supports = []
    for support_id in G.support_map:
        support = G.support_map[support_id]
        support["@id"] = support_id
        supports.append(support)
    return [{"supports": supports}]


def node_supports(G):
    node_supports = []
    for node_id in G.node_support_map:
        supports = G.node_support_map[node_id]
        node_supports.append({"supports": supports, "po": [node_id]})
    return [{"nodeSupports": node_supports}]


def edge_supports(G):
    edge_supports = []
    for edge_id in G.edge_support_map:
        supports = G.edge_support_map[edge_id]
        edge_supports.append({"supports": supports, "po": [edge_id]})
    return [{"edgeSupports": edge_supports}]

def function_terms(G):
    function_terms = []
    for po in G.function_term_map:
        function_terms.append(G.function_term_map[po])
    return [{"functionTerms": function_terms}]

def reified_edges(G):
    reified_edges = []
    for n,re in G.reified_edges.iteritems():
        reified_edges.append(re)
    return [{"reifiedEdges": reified_edges}]

def provenance(G):
    if G.get_provenance():
        return [{"provenanceHistory": [G.get_provenance()]}]
    else:
        return []

def namespaces(G):
    if G.namespaces:
        return [{'@context': [G.namespaces]}]
    else :
        return []

# cv stands for convert value. This converts a type to a string representation for CX purposes.
def cv(val):
    return val


def domain(val):
    if type(val) is list:
        if isinstance(val[0], string_types):
            return 'list_of_string'
        elif type(val[0]) is bool:
            return 'list_of_boolean'
        elif isinstance(val[0], integer_types):
            if PY3:
                return 'list_of_integer'
            else:
                if type(val[0]) is int:
                    return 'list_of_integer'
                else:
                    return 'list_of_long'
        elif type(val[0]) is float:
            return 'list_of_double'
        else:
            return 'list_of_unknown'

    if isinstance(val, string_types):
        return 'string'
    elif type(val) is bool:
        return 'boolean'
    elif type(val) is int:
        return 'integer'
    elif isinstance(val, integer_types):
        if PY3:
                return 'integer'
        else:
            if type(val) is int:
                return 'integer'
            else:
                return 'long'
    elif type(val) is float:
        return 'double'
    else:
        return 'unknown'



__author__ = 'aarongary'

import unittest

import sys
import time
import ijson
import pickle
import json
from nicecxModel.NiceCXNetwork import NiceCXNetwork
from nicecxModel.cx.aspects.NodesElement import NodesElement
from nicecxModel.cx.aspects.EdgesElement import EdgesElement
from nicecxModel.cx.aspects.NetworkAttributesElement import NetworkAttributesElement
from nicecxModel.cx.aspects.NodeAttributesElement import NodeAttributesElement
from nicecxModel.cx.aspects.EdgeAttributesElement import EdgeAttributesElement
from nicecxModel.cx.aspects.CitationElement import CitationElement
from nicecxModel.cx.aspects.SupportElement import SupportElement
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx.aspects.SimpleNode import SimpleNode
from nicecxModel.cx import CX_CONSTANTS
from ndex2.NetworkQuery import NetworkQuery
from ndex2.client import Ndex2
from ndex2.NiceCXBuilder import NiceCXBuilder

if sys.version_info.major == 3:
    from urllib.request import urlopen
else:
    from urllib import urlopen


def get_nodes():
    for number in range(0, 10000):
        yield_this = SimpleNode(id=str(number), node_name='Node' + str(number), node_represents='Node' + str(number))
        #yield_this = NodesElement(id=str(number), node_name='Node' + str(number), node_represents='Node' + str(number))
        #yield_this = {'@id': str(number), 'n': 'Node' + str(number), 'r': 'Node' + str(number)}
        yield yield_this

if False:
    node_getter = get_nodes()

    node_array = []

    for node in node_getter:
        node_array.append(node)

    serialized = pickle.dumps(node_array, protocol=0)
    print('Serialized memory:', sys.getsizeof(serialized))


    print(len(node_array))


def loadAspect(aspect_name):
    #with open('Signal1.cx', mode='r') as cx_f:
    with open('network1.cx', mode='r') as cx_f:
        aspect_json = json.loads(cx_f.read())
        for aspect in aspect_json:
            if aspect.get(aspect_name) is not None:
                return aspect.get(aspect_name)


class MyTestCase(unittest.TestCase):
    def test_ndex_load_cx_model(self):
        niceCxBuilder = NiceCXBuilder()
        nice_cx_from_builder = niceCxBuilder.create_from_server('http://dev2.ndexbio.org', 'scratch', 'scratch', '94766028-934d-11e7-9743-0660b7976219')
        print(nice_cx_from_builder)
        #ndex = Ndex(server='http://dev2.ndexbio.org', username='scratch', password='scratch', uuid='94766028-934d-11e7-9743-0660b7976219')
        #cx = ndex.get_network_as_cx_stream(uuid).json()
        #if not cx:


    @unittest.skip("Temporary skipping")
    def test_nice_cx_model(self):

        niceCx = NiceCXNetwork()
        #main_map = NdexGraph(server='http://dev2.ndexbio.org', username='scratch', password='scratch', uuid='7246d8cf-c644-11e6-b48c-0660b7976219')

        uuid = '6b968fd2-02a4-11e6-b550-06603eb7f303'

        #====================
        # NETWORK QUERY
        #====================

        networkQuery = NetworkQuery()
        networkQuery.query_network('40f1def0-3aa4-11e7-b12f-0660b7976219', 'HSPA5,HSPA4')

        my_na = NodeAttributesElement(subnetwork=1, property_of=11, name=22, values=33, type=ATTRIBUTE_DATA_TYPE.convert_to_data_type('string'))

        #====================
        # NETWORK ATTRIBUTES
        #====================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/nodes'), 'item')
        objects = loadAspect('networkAttributes')
        obj_items = (o for o in objects)
        for network_item in obj_items:
            add_this_network_attribute = NetworkAttributesElement(json_obj=network_item)

            niceCx.addNetworkAttribute(add_this_network_attribute)

        #===================
        # NODES
        #===================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/nodes'), 'item')
        objects = loadAspect('nodes')
        obj_items = (o for o in objects)
        for node_item in obj_items:
            add_this_node = NodesElement(json_obj=node_item)

            niceCx.addNode(add_this_node)

        #===================
        # EDGES
        #===================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/edges'), 'item')

        objects = loadAspect('edges')
        obj_items = (o for o in objects)
        for edge_item in obj_items:
            add_this_edge = EdgesElement(json_obj=edge_item)

            niceCx.addEdge(add_this_edge)

        #===================
        # NODE ATTRIBUTES
        #===================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/nodeAttributes'), 'item')
        objects = loadAspect('nodeAttributes')
        obj_items = (o for o in objects)
        for att in obj_items:
            add_this_node_att = NodeAttributesElement(json_obj=att)

            niceCx.addNodeAttribute(add_this_node_att)

        #===================
        # EDGE ATTRIBUTES
        #===================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/edgeAttributes'), 'item')
        objects = loadAspect('edgeAttributes')
        obj_items = (o for o in objects)
        for att in obj_items:
            add_this_edge_att = EdgeAttributesElement(json_obj=att)

            niceCx.addEdgeAttribute(add_this_edge_att)

        #===================
        # CITATIONS
        #===================
        #objects = ijson.items(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/edgeAttributes'), 'item')
        objects = loadAspect('citations')
        obj_items = (o for o in objects)
        for cit in obj_items:
            add_this_citation = CitationElement(json_obj=cit)

            niceCx.addCitation(add_this_citation)

        #===================
        # SUPPORTS
        #===================
        objects = loadAspect('supports')
        obj_items = (o for o in objects)
        for sup in obj_items:
            add_this_supports = SupportElement(json_obj=sup)

            niceCx.addSupport(add_this_supports)

        #===================
        # NODE CITATIONS
        #===================
        objects = loadAspect('nodeCitations')
        obj_items = (o for o in objects)
        for node_cit in obj_items:
            niceCx.addNodeCitationsFromCX(node_cit)

        #===================
        # EDGE CITATIONS
        #===================
        objects = loadAspect('edgeCitations')
        obj_items = (o for o in objects)
        for edge_cit in obj_items:
            niceCx.addEdgeCitationsFromCX(edge_cit)

        nice_cx_json = niceCx.to_json()

#        serialized = pickle.dumps(niceCx, protocol=0)
#        print 'Serialized memory:', sys.getsizeof(serialized)


        print('starting to_pandas')
        niceCx.to_pandas()
        parser = ijson.parse(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/nodes'))

        node_id, node_n, node_r = '', '', ''
        edge_id, edge_s, edge_t, edge_i = '', '', '', ''
        node_matches = {}
        edge_matches = {}
        edge_connected = {}
        node_found = False
        edge_found = False
        count = 0
        done_searching = False

        start_time = time.time()

        for prefix, event, value in parser:
            if (prefix) == ('item.@id'):
                if count % 10000 == 0:
                    print(count)
                count += 1
                node_id = value
            elif (prefix) == ('item.n'):
                node_n = value
                node_found = True
            elif (prefix) == ('item.r'):
                node_r = value
                if node_found:
                    node_matches[node_id] = {'n': node_n, 'r': node_r}
                    add_this_node = NodesElement(id=node_id, node_name=node_n, node_represents=node_r)
                    niceCx.addNode(add_this_node)
                    node_found = False
            else:
                # No represents found
                if node_found:
                    node_matches[node_id] = {'n': node_n}
                    add_this_node = NodesElement(id=node_id, node_name=node_n)
                    niceCx.addNode(add_this_node)
                    node_found = False

        print('Response time (Node search): ' + str(time.time() - start_time))
        start_time = time.time()

        print(edge_matches)
        print(node_matches)

        parser = ijson.parse(urlopen('http://dev2.ndexbio.org/v2/network/' + uuid + '/aspect/edges'))

        for prefix, event, value in parser:
            if (prefix) == ('item.@id'):
                edge_id = value
            elif (prefix) == ('item.s'):
                edge_s = value
                edge_found = True
            elif (prefix) == ('item.t'):
                edge_t = value
                edge_found = True
            elif (prefix) == ('item.i'):
                edge_i = value
                if edge_found:
                    edge_matches[edge_id] = {'s': edge_s, 't': edge_t, 'i': edge_i}
                    add_this_edge = EdgesElement(id=edge_id, edge_source=edge_s, edge_target=edge_t, edge_interaction=edge_i)
                    niceCx.addEdge(add_this_edge)
                    edge_connected[edge_s] = 1
                    edge_connected[edge_t] = 1
                    edge_found = False
            else:
                # No interaction found
                if edge_found:
                    edge_matches[edge_id] = {'s': edge_s, 't': edge_t}
                    add_this_edge = EdgesElement(id=edge_id, edge_source=edge_s, edge_target=edge_t)
                    niceCx.addEdge(add_this_edge)
                    edge_connected[edge_s] = 1
                    edge_connected[edge_t] = 1
                    edge_found = False

        print('Response time (Edge search): ' + str(time.time() - start_time))
        start_time = time.time()

        print(edge_matches)
        print(node_matches)

        self.assertTrue(niceCx is not None)





import unittest

import ndex2.client as nc
import time
from os import path
import ndex2.test.test_NdexClient as tt

ndex_network_resource = "/v2/network/"

example_network_1 = 'A549-SL-network.cx'

# Python Client APIs tested:
#
#   get_user_by_username
#


class MyTestCase(unittest.TestCase):

    def test_get_user_by_username(self):
        ndex = nc.Ndex2(host=tt.TESTSERVER, username=tt.testUser1, password=tt.testUserpasswd, debug=True)

        with open(path.join(path.abspath(path.dirname(__file__)),example_network_1), 'r') as file_handler:
            network_in_cx = file_handler.read()

        # test save_cx_stream_as_new_network
        test_network_1_uri = ndex.save_cx_stream_as_new_network(network_in_cx)
        self.assertTrue(test_network_1_uri.startswith(tt.TESTSERVER + ndex_network_resource))

        network_UUID = str(test_network_1_uri.split("/")[-1])

        # get network summary
        network_summary = ndex.get_network_summary(network_UUID)
        network_visibility = str(network_summary['visibility'])
        network_read_only  =  network_summary['isReadOnly']
        self.assertTrue(network_visibility.upper() == 'PRIVATE')
        self.assertTrue(network_read_only == False)


        # test set_network_system_properties
        time.sleep(10)


        # make network ReadOnly and PUBLIC
        ndex.set_network_system_properties(network_UUID, {'readOnly': True})
        ndex.set_network_system_properties(network_UUID, {'visibility': 'PUBLIC'})

        # check if we succeeded in making the netwrk ReadOnly and PUBLIC
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PUBLIC')
        self.assertTrue(network_summary['isReadOnly'] == True)

        # make network Read-Write and PRIVATE
        ndex.set_network_system_properties(network_UUID, {'readOnly': False})
        ndex.set_network_system_properties(network_UUID, {'visibility': 'PRIVATE'})
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PRIVATE')
        self.assertTrue(network_summary['isReadOnly'] == False)



        # make network ReadOnly and PUBLIC in one call
        ndex.set_network_system_properties(network_UUID, {'readOnly': True, 'visibility': 'PUBLIC'})
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PUBLIC')
        self.assertTrue(network_summary['isReadOnly'] == True)

        # make network ReadWrite and PRIVATE in one call
        ndex.set_network_system_properties(network_UUID, {'readOnly': False, 'visibility': 'PRIVATE'})
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PRIVATE')
        self.assertTrue(network_summary['isReadOnly'] == False)

        # make network ReadOnly and PRIVATE in one call
        ndex.set_network_system_properties(network_UUID, {'readOnly': True, 'visibility': 'PRIVATE'})
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PRIVATE')
        self.assertTrue(network_summary['isReadOnly'] == True)

        # make network ReadWrite and PUBLIC in one call
        ndex.set_network_system_properties(network_UUID, {'readOnly': False, 'visibility': 'PUBLIC'})
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(str(network_summary['visibility']).upper() == 'PUBLIC')
        self.assertTrue(network_summary['isReadOnly'] == False)



        # test delete_network
        del_network_return = ndex.delete_network(network_UUID)
        self.assertTrue(del_network_return == '')

if __name__ == '__main__':
    unittest.main()




import unittest

import ndex2.client as nc
import time
from os import path


ndex_host = "http://dev.ndexbio.org"
ndex_network_resource = "/v2/network/"
username_1 = "ttt"
password_1 = "ttt"

example_network_1 = 'A549-SL-network.cx'

# Python Client APIs tested:
#
#   save_cx_stream_as_new_network
#   get_network_as_cx_stream
#   delete_network
#


class MyTestCase(unittest.TestCase):

    def test_get_network_as_cx_stream(self):
        ndex = nc.Ndex2(host=ndex_host, username=username_1, password=password_1)

        with open(path.join(path.abspath(path.dirname(__file__)),example_network_1), 'r') as file_handler:
            network_in_cx_from_file = file_handler.read()

        # test save_cx_stream_as_new_network
        test_network_1_uri = ndex.save_cx_stream_as_new_network(network_in_cx_from_file)
        self.assertTrue(test_network_1_uri.startswith(ndex_host + ndex_network_resource))

        network_UUID = str(test_network_1_uri.split("/")[-1])

        network_as_cx_stream = ndex.get_network_as_cx_stream(network_UUID)
        network_as_cx = str(network_as_cx_stream.text)

        self.assertTrue(network_in_cx_from_file == network_as_cx)

        time.sleep(10)


        # test delete_network
        del_network_return = ndex.delete_network(network_UUID)
        self.assertTrue(del_network_return == '')



if __name__ == '__main__':
    unittest.main()




import unittest

import ndex2.client as nc

import json
import ndex2.test.test_NdexClient as tt


# Python Client APIs tested:
#
#   save_cx_stream_as_new_network
#   get_neighborhood_as_cx_stream
#   get_neighborhood
#   delete_network
#


class MyTestCase(tt.NdexClientTestCase):

    @classmethod
    def getNumberOfNodesAndEdgesFromCX(self, network_in_cx):
        for aspect in network_in_cx:
            if 'metaData' in aspect:
                metaData = aspect['metaData']

                for element in metaData:
                    if (('name' in element) and (element['name'] == 'nodes')):
                        numberOfNodes = element['elementCount']

                    if (('name' in element) and (element['name'] == 'edges')):
                        numberOfEdges = element['elementCount']

                break

        return numberOfNodes, numberOfEdges


    def test_get_neighborhood_as_cx_stream(self):

        search_string = 'AANAT-T3M'
        # run neighborhood query with depth 1; expected to receive subnetwork with 2 edges and 3 nodes
        network_neighborhood = self._ndex.get_neighborhood_as_cx_stream(self._networkId, search_string, 1)
        self.assertTrue(network_neighborhood.status_code == 200)
        network_in_cx = json.loads(network_neighborhood.text)

        numberOfNodes, numberOfEdges = self.getNumberOfNodesAndEdgesFromCX(network_in_cx['data'])
        self.assertTrue(numberOfNodes == 5)
        self.assertTrue(numberOfEdges == 6)


        # now run neighborhood query with depth 2; expected to receive subnetwork with 6 edges and 5 nodes
        network_neighborhood = self._ndex.get_neighborhood_as_cx_stream(self._networkId, search_string, 2)
        self.assertTrue(network_neighborhood.status_code == 200)
        network_in_cx = json.loads(network_neighborhood.text)

        numberOfNodes, numberOfEdges = self.getNumberOfNodesAndEdgesFromCX(network_in_cx['data'])
        self.assertTrue(numberOfNodes == 5)
        self.assertTrue(numberOfEdges == 6)

        # run neighborhood query with depth 1; expected to receive subnetwork with 2 edges and 3 nodes
        network_neighborhood = self._ndex.get_neighborhood(self._networkId, search_string, 1)
        numberOfNodes, numberOfEdges = self.getNumberOfNodesAndEdgesFromCX(network_neighborhood)
        self.assertTrue(numberOfNodes == 5)
        self.assertTrue(numberOfEdges == 6)


        # now run neighborhood query with depth 2; expected to receive subnetwork with 6 edges and 5 nodes
        network_neighborhood = self._ndex.get_neighborhood(self._networkId, search_string, 2)
        numberOfNodes, numberOfEdges = self.getNumberOfNodesAndEdgesFromCX(network_neighborhood)
        self.assertTrue(numberOfNodes == 5)
        self.assertTrue(numberOfEdges == 6)


if __name__ == '__main__':
    unittest.main()




import unittest

import ndex2.client as nc
import time
from os import path
import ndex2.test.test_NdexClient as tt

ndex_network_resource = "/v2/network/"

example_network_1 = 'A549-SL-network.cx'


# Python Client APIs tested:
#
#   set_network_properties
#


class MyTestCase(unittest.TestCase):

    def test_network_properties(self):
        ndex = nc.Ndex2(host= tt.TESTSERVER, username=tt.testUser1, password=tt.testUserpasswd, debug=True)

        with open(path.join(path.abspath(path.dirname(__file__)),example_network_1), 'r') as file_handler:
            network_in_cx = file_handler.read()

        # test save_cx_stream_as_new_network
        test_network_1_uri = ndex.save_cx_stream_as_new_network(network_in_cx)
        self.assertTrue(test_network_1_uri.startswith(tt.TESTSERVER + ndex_network_resource))

        network_UUID = str(test_network_1_uri.split("/")[-1])


        ################### first, we add a new property with subNetworkId == None ###################

        # get network summary
        network_summary = ndex.get_network_summary(network_UUID)

        property_list = []
        if "properties" in network_summary:
            property_list = network_summary['properties']

        property_list.append({
            'dataType' : 'string',
            'predicateString' : 'newProperty',
            'subNetworkId' : None,
            'value' : 'New Value'
        })

        time.sleep(10)
        number_of_properties = ndex.set_network_properties(network_UUID, property_list)
        self.assertTrue(number_of_properties == len(property_list))

        # get network summary with the new properties
        network_summary_1 = ndex.get_network_summary(network_UUID)
        property_list_1 = network_summary_1['properties']
        self.assertTrue(property_list == property_list_1)




        ################### now, we add a new property with subNetworkId != None ###################

        self.assertTrue("subnetworkIds" in network_summary, "'subnetworkIds' structure is not in network_summary")

        number_of_subnetworks = len(network_summary["subnetworkIds"])
        self.assertTrue(number_of_subnetworks == 1, "Expected 1 subnetwork in network summary, but there are " \
                        + str(number_of_subnetworks))

        subnetwork_id = network_summary["subnetworkIds"][0]

        property_list_1.append({
            'dataType' : 'string',
            'predicateString' : 'newProperty',
            'subNetworkId' : subnetwork_id,
            'value' : 'New Value'
        })

        number_of_properties = ndex.set_network_properties(network_UUID, property_list_1)
        self.assertTrue(number_of_properties == len(property_list_1))

        # get network summary with the new properties
        network_summary_2 = ndex.get_network_summary(network_UUID)
        property_list_2 = network_summary_2['properties']
        self.assertTrue(property_list_1 == property_list_2)



        # test delete_network
        del_network_return = ndex.delete_network(network_UUID)
        self.assertTrue(del_network_return == '')

if __name__ == '__main__':
    unittest.main()




__author__ = 'aarongary'
from enum import Enum
from six import string_types, integer_types
from sys import version_info
PY3 = version_info > (3,)


CX_CONSTANTS = {
    'ID': '@id'
}

class ATTRIBUTE_DATA_TYPE(Enum):
    BOOLEAN = 'boolean'
    BYTE = 'byte'
    CHAR = 'char'
    DOUBLE = 'double'
    FLOAT = 'float'
    INTEGER = 'integer'
    LONG = 'long'
    SHORT = 'short'
    STRING = 'string'
    LIST_OF_BOOLEAN = 'list_of_boolean'
    LIST_OF_BYTE = 'list_of_byte'
    LIST_OF_CHAR = 'list_of_char'
    LIST_OF_DOUBLE = 'list_of_double'
    LIST_OF_FLOAT = 'list_of_float'
    LIST_OF_INTEGER = 'list_of_integer'
    LIST_OF_LONG = 'list_of_long'
    LIST_OF_SHORT = 'list_of_short'
    LIST_OF_STRING = 'list_of_string'

    @classmethod
    def toCxLabel(self, dt):
        if dt == ATTRIBUTE_DATA_TYPE.BOOLEAN:
            return 'boolean'
        elif dt == ATTRIBUTE_DATA_TYPE.BYTE:
            return 'byte'
        elif dt == ATTRIBUTE_DATA_TYPE.CHAR:
            return 'char'
        elif dt == ATTRIBUTE_DATA_TYPE.DOUBLE:
            return 'double'
        elif dt == ATTRIBUTE_DATA_TYPE.FLOAT:
            return 'float'
        elif dt == ATTRIBUTE_DATA_TYPE.INTEGER:
            return 'integer'
        elif dt == ATTRIBUTE_DATA_TYPE.LONG:
            return 'long'
        elif dt == ATTRIBUTE_DATA_TYPE.SHORT:
            return 'short'
        elif dt == ATTRIBUTE_DATA_TYPE.STRING:
            return 'string'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_BOOLEAN:
            return 'list_of_boolean'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_BYTE:
            return 'list_of_byte'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_CHAR:
            return 'list_of_char'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_DOUBLE:
            return 'list_of_double'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_FLOAT:
            return 'list_of_float'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_INTEGER:
            return 'list_of_integer'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_LONG:
            return 'list_of_long'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_SHORT:
            return 'list_of_short'
        elif dt == ATTRIBUTE_DATA_TYPE.LIST_OF_STRING:
            return 'list_of_string'
        else:
            raise Exception('don\'t know type ' + dt)

    @classmethod
    def fromCxLabel(self, s):
        if s == 'boolean':
            return ATTRIBUTE_DATA_TYPE.BOOLEAN
        elif s == 'byte':
            return ATTRIBUTE_DATA_TYPE.BYTE
        elif s == 'char':
            return ATTRIBUTE_DATA_TYPE.CHAR
        elif s == 'double':
            return ATTRIBUTE_DATA_TYPE.DOUBLE
        elif s == 'float':
            return ATTRIBUTE_DATA_TYPE.FLOAT
        elif s == 'integer':
            return ATTRIBUTE_DATA_TYPE.INTEGER
        elif s == 'long':
            return ATTRIBUTE_DATA_TYPE.LONG
        elif s == 'short':
            return ATTRIBUTE_DATA_TYPE.SHORT
        elif s == 'string':
            return ATTRIBUTE_DATA_TYPE.STRING
        elif s == 'list_of_boolean':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_BOOLEAN
        elif s == 'list_of_byte':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_BYTE
        elif s == 'list_of_char':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_CHAR
        elif s == 'list_of_double':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_DOUBLE
        elif s == 'list_of_float':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_FLOAT
        elif s == 'list_of_integer':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_INTEGER
        elif s == 'list_of_long':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_LONG
        elif s == 'list_of_short':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_SHORT
        elif s == 'list_of_string':
            return ATTRIBUTE_DATA_TYPE.LIST_OF_STRING
        else:
            raise Exception('don\'t know type ' + s)

    @classmethod
    def isSingleValueType(self, dt):
        if dt in [ATTRIBUTE_DATA_TYPE.BOOLEAN, ATTRIBUTE_DATA_TYPE.BYTE, ATTRIBUTE_DATA_TYPE.CHAR,
                  ATTRIBUTE_DATA_TYPE.DOUBLE, ATTRIBUTE_DATA_TYPE.FLOAT, ATTRIBUTE_DATA_TYPE.INTEGER,
                  ATTRIBUTE_DATA_TYPE.LONG, ATTRIBUTE_DATA_TYPE.SHORT, ATTRIBUTE_DATA_TYPE.STRING]:
            return True
        else:
            return False

    @classmethod
    def convert_to_data_type(self, val):
        if type(val) is list:
            if isinstance(val[0], string_types):
                return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_string')
            elif type(val[0]) is bool:
                return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_boolean')
            elif isinstance(val[0], integer_types):
                if PY3:
                    return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_integer')
                else:
                    if type(val[0]) is int:
                        return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_integer')
                    else:
                        return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_long')
            elif type(val[0]) is float:
                return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_double')
            else:
                return ATTRIBUTE_DATA_TYPE.fromCxLabel('list_of_string') #'list_of_unknown')

        if isinstance(val, string_types):
            return ATTRIBUTE_DATA_TYPE.fromCxLabel('string')
        elif type(val) is bool:
            return ATTRIBUTE_DATA_TYPE.fromCxLabel('boolean')
        elif type(val) is int:
            return ATTRIBUTE_DATA_TYPE.fromCxLabel('integer')
        elif isinstance(val, integer_types):
            if PY3:
                return ATTRIBUTE_DATA_TYPE.fromCxLabel('integer')
            else:
                if type(val) is int:
                    return ATTRIBUTE_DATA_TYPE.fromCxLabel('integer')
                else:
                    return ATTRIBUTE_DATA_TYPE.fromCxLabel('long')
        elif type(val) is float:
            return ATTRIBUTE_DATA_TYPE.fromCxLabel('double')
        else:
            return ATTRIBUTE_DATA_TYPE.fromCxLabel('string')  #'unknown')




__author__ = 'aarongary'

from nicecxModel.cx import CX_CONSTANTS
import json

class SupportElement(object):
    def __init__(self, id=None, text=None, citation_id=None, attributes=None, props=None, json_obj=None):
        self.ASPECT_NAME = 'supports'

        if json_obj is None:
            self._text = text
            self._id = id
            self._citation_id = citation_id
            self._attributes = attributes
            self._props = props
        else:
            self._text = json_obj.get(CX_CONSTANTS.TEXT.value)
            self._id = json_obj.get(CX_CONSTANTS.ID.value)
            self._citation_id = json_obj.get(CX_CONSTANTS.CITATION.value)
            self._attributes = json_obj.get(CX_CONSTANTS.ATTRIBUTES.value)
            self._props = json_obj.get(CX_CONSTANTS.PROPERTIES.value)

        self.support_element = {}

    def getText(self):
        return self._text

    def setText(self, text):
        self._text = text

    def getId(self):
        return self._id

    def setId(self, id):
        self._id = id

    def getCitationId(self):
        return self._citation_id

    def setCitationId(self, citation_id):
        self._citation_id = citation_id

    def getAttribute(self):
        return self._attributes

    def setAttributes(self, attibrutes):
        self._attributes = attibrutes

    def getProps(self):
        return self._props

    def setProps(self, props):
        self._props = props

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        return_json = {}

        if self._id == -1:
            raise Exception('Edge element does not have a valid ID.  Unable to process this edge - ' + self._node_name)

        return_json[CX_CONSTANTS.ID.value] = self._id

        if self._text is not None:
            return_json[CX_CONSTANTS.TEXT.value] = self._text

        if self._citation_id is not None:
            return_json[CX_CONSTANTS.CITATION_IDENTIFIER.value] = self._citation_id

        if self._attributes is not None and len(self._attributes) > 0:
            return_json[CX_CONSTANTS.ATTRIBUTES.value] = self._attributes

        if self._props is not None and len(self._props) > 0:
            return_json[CX_CONSTANTS.PROPERTIES.value] = self._props

        return return_json




__author__ = 'aarongary'

import json

class SimpleNode(object):
    def __init__(self, id=None, node_name=None, node_represents=None):
        self.return_this = {
            'n': node_name,
            '@id': id,
            'r': node_represents
        }

    def __str__(self):
        return json.dumps(self.return_this)



import os
import logging
import logging.handlers

root = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
log_path = os.path.join(root, 'logs')
#solr_url = 'http://localhost:8983/solr/'
solr_url = 'http://dev2.ndexbio.org:8983/solr/'
deprecation_message = 'This function is now deprecated. use NiceCX'

def get_logger(name, level=logging.DEBUG):
    logger = logging.getLogger(name)
    logger.setLevel(level)

    logger.handlers = []

    formatter = logging.Formatter('%(asctime)s.%(msecs)d ' + name + ' %(levelname)s: %(message)s', '%Y-%m-%d %H:%M:%S')

    handler = logging.handlers.TimedRotatingFileHandler(os.path.join(log_path, 'app.log'), when='midnight', backupCount=28)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger




import unittest
from os import path
import json
import ndex2.beta.layouts as layouts
from ndex2.networkn import NdexGraph


HERE = path.abspath(path.dirname(__file__))

class NetworkNConstructorTests(unittest.TestCase):

    def test1(self):
        with open(path.join(path.abspath(path.dirname(__file__)), 'tiny_corpus.cx'),'r') as cx_file:
            G = NdexGraph(cx=json.load(cx_file))
            '''self.assertEqual(len(G.edgemap),37 )
            self.assertEqual(len(G.node), 37)
            self.assertEqual(G.provenance['entity']['properties'][0]['name'], 'edge count')
            self.assertEqual(G.provenance['entity']['properties'][0]['value'], '37')
            self.assertEqual(len(G.support_map), 15)
            self.assertEqual(len(G.citation_map), 1)
            self.assertEqual(len(G.function_term_map),35)
            self.assertEqual(len(G.node_citation_map),0)
            self.assertEqual(len(G.node_support_map), 0)
            self.assertEqual(len(G.node_citation_map),0)
            self.assertEqual(len(G.reified_edges), 2)
            self.assertEqual(len(G.edge_citation_map),37)
            self.assertEqual(len(G.edge_support_map),37)
            self.assertEqual(len(G.namespaces),39)'''

            G.update_to('b8fedefb-29e5-11e7-b3a1-06832d634f41', 'http://dev.ndexbio.org', 'scratch', 'scratch')
        print "done"

    def test2(self):
        with open (path.join(HERE,'filtered.cx'),'r') as cx_file:
            cx=json.load(cx_file)
            g = NdexGraph(cx)
            layouts.apply_directed_flow_layout(g)
            self.assertEqual(g.node[80]['diffusion_input'], 1.0)


if __name__ == '__main__':
    unittest.main()


from __future__ import absolute_import

import unittest
from os import path
import ndex2.client as nc
import uuid
import time
import json
import ndex2.test.test_NdexClient as tt

class NdexClientTestCase4(tt.NdexClientRASMachineTestCase):

    def testNetworkVisibility(self):
        ndex2 = nc.Ndex2(tt.TESTSERVER)
        with self.assertRaises(Exception) as context:
            ndex2.get_network_summary(self._networkId)
#        print context.exception
        count = 0
        while count < 30 :
            try :
                self._ndex.make_network_public(self._networkId)
                break
            except Exception as inst :
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification"):
                    print("retry in 5 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(5)
                else :
                    raise inst

        summary = ndex2.get_network_summary(self._networkId)
        self.assertEqual(summary.get(u'externalId'), self._networkId)
        print("make_network_public() passed.")

        time.sleep(1)


        count = 0
        while count < 30:
            try:
                self._ndex.make_network_private(self._networkId)
                break
            except Exception as inst:
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification"):
                    print("retry in 5 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(5)
                else:
                    raise inst

        with self.assertRaises(Exception) as context:
            ndex2.get_network_summary(self._networkId)
        print("make_network_private() passed.")

if __name__ == '__main__':
    unittest.main()



import unittest

import ndex2.client as nc
import time
from os import path
import ndex2.test.test_NdexClient as tt


ndex_network_resource = "/v2/network/"

example_network_1 = 'A549-SL-network.cx'

# Python Client APIs tested:
#
#   save_cx_stream_as_new_network
#   get_provenance
#   set_provenance
#   update_network_profile
#   get_network_summary
#   delete_network
#


class MyTestCase(unittest.TestCase):
    def test_provenance_and_profile(self):
        ndex = nc.Ndex2(host=tt.TESTSERVER, username=tt.testUser1, password=tt.testUserpasswd, debug=True)

        with open(path.join(path.abspath(path.dirname(__file__)), example_network_1), 'r') as file_handler:
            network_in_cx = file_handler.read()


        # test save_cx_stream_as_new_network
        test_network_1_uri = ndex.save_cx_stream_as_new_network(network_in_cx)
        self.assertTrue(test_network_1_uri.startswith(tt.TESTSERVER + ndex_network_resource))


        # test get_provenance
        network_UUID = str(test_network_1_uri.split("/")[-1])
        provenance = ndex.get_provenance(network_UUID)
        self.assertTrue(provenance)
        provenance_keys = provenance.keys()
        # check that fields creationEvent, properties and uri are fields in the provenance structure
        self.assertTrue('creationEvent' in provenance_keys)
        self.assertTrue('properties' in provenance_keys)
        self.assertTrue('uri' in provenance_keys)
        uri1 = str(provenance['uri']);
        uri2 = tt.TESTSERVER + ndex_network_resource + network_UUID + "/summary"
        self.assertTrue(uri1 == uri2)


        # test update_network_profile
        new_version = "1.55"
        new_name = "New network name"
        new_description = "New network description"
        test_profile = {"version": new_version,
                        "name": new_name,
                        "description": new_description
                        }
        time.sleep(10)
        ndex.update_network_profile(network_UUID, test_profile)

        # test get_network_summary
        network_summary = ndex.get_network_summary(network_UUID)
        self.assertTrue(new_version == network_summary['version'])
        self.assertTrue(new_name == network_summary['name'])
        self.assertTrue(new_description == network_summary['description'])

        time.sleep(10)
        test_profile['visibility'] = 'PUBLIC'
        with self.assertRaises(Exception) as e:
            ndex.update_network_profile(network_UUID, test_profile)


      #  errorMessage = str(e)
      #  print errorMessage
      #  self.assertTrue(errorMessage.startswith("Ndex 2.x doesn't support setting visibility by this function."))

        provenance_1 = ndex.get_provenance(network_UUID)

        # test set_provenance - change description of event
        provenance_1['creationEvent']['eventType'] = 'Modified Program Upload in CX'
        ndex.set_provenance(network_UUID, provenance_1)
        provenance_2 = ndex.get_provenance(network_UUID)
        self.assertTrue(provenance_2['creationEvent']['eventType'] == provenance_1['creationEvent']['eventType'])

        # set provenance back to the original one
        ndex.set_provenance(network_UUID, provenance)
        provenance_3 = ndex.get_provenance(network_UUID)
        self.assertTrue(provenance['creationEvent']['eventType'] == provenance_3['creationEvent']['eventType'])

        # test delete_network
        del_network_return = ndex.delete_network(network_UUID)
        self.assertTrue(del_network_return == '')

if __name__ == '__main__':
    unittest.main()




from __future__ import absolute_import

import unittest
from os import path
import ndex2.client as nc
import uuid
import time
import json
import ndex2.test.test_NdexClient as tt
#from ndex.networkn import NdexGraph


class NdexClientTestCase(tt.NdexClientTestCase):

    def testUpdateNetworkNSetProperty(self):
        count = 0
        while count < 30 :
            try :
                with open(path.join(tt.HERE, 'The_RAS_Machine.cx'), 'r') as cx_file:
                    self._ndex.update_cx_network(cx_file, self._networkId)
                break

            except Exception as inst :
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification"):
                    print("retry in 1 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(2)
                else :
                    raise inst

        summary = self._ndex.get_network_summary(self._networkId)
        self.assertEqual(summary.get(u'externalId'), self._networkId)

        profile = {'name': "Rudi's updated network",
                   'description': 'nice_description',
                   'version': 'new_version'}

        #time.sleep(6)
        count = 0
        while count < 30:
            try:
                self._ndex.update_network_profile(self._networkId,profile)
                break
            except Exception as inst:
                d = json.loads(inst.response.content)
                if d.get('errorCode').startswith("NDEx_Concurrent_Modification"):
                    print("retry in 1 seconds(" + str(count) + ")")
                    count += 1
                    time.sleep(2)
                else:
                    raise inst

        summary =  self._ndex.get_network_summary(self._networkId)
        self.assertTrue(summary['name'], profile['name'])
        cx = self._ndex.get_network_as_cx_stream(self._networkId).json()
        #G = NdexGraph(cx=cx)
        #self.assertEqual(len(G.edgemap),1574)
        #self.assertEqual(len(G.node),513)
 #       print G.metadata_original

        print("Update network and setProperty test passed.")

if __name__ == '__main__':
    unittest.main()



__author__ = 'aarongary'


import json
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx import CX_CONSTANTS

class CitationCommon(object):
    def __init__(self, subnetwork=None, citations=None, property_of=None, json_obj=None):
        if json_obj is not None:
            self._property_of = json_obj.get(CX_CONSTANTS.PROPERTY_OF)
            self._subnetwork = json_obj.get(CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK)
            self._citations = json_obj.get(CX_CONSTANTS.CITATIONS)
        else:
            self._property_of = property_of
            self._subnetwork = subnetwork
            self._citations = citations

        self.ASPECT_NAME = None

    def getPropertyOf(self):
        return self._property_of

    def setPropertyOf(self, id):
        self._property_of = id

    def getSubnetwork(self):
        return self._subnetwork

    def setSubnetwork(self, subnetwork):
        self._subnetwork = subnetwork

    def getCitations(self):
        return self._citations

    def setCitations(self, citations):
        self._citations = citations

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        return_dict = {}

        if self._property_of is not None:
            return_dict[CX_CONSTANTS.PROPERTY_OF] = self._property_of

        if self._subnetwork:
            return_dict[CX_CONSTANTS.EDGE_SOURCE_NODE_ID_OR_SUBNETWORK] = self._subnetwork

        if self._citations:
            return_dict[CX_CONSTANTS.CITATIONS] = self._citations

        return return_dict


__author__ = 'aarongary'

import codecs
import mimetypes
import sys
import uuid
import requests
import json
from requests_toolbelt import MultipartEncoder
try:
    import io
except ImportError:
    pass # io is requiered in python3 but not available in python2

class MultipartCXEncoder2(MultipartEncoder):
    def __init__(self, fields, nice_cx):
        #self.boundary = uuid.uuid4().hex
        #self.content_type = 'multipart/form-data; boundary={}'.format(self.boundary)
        super(MultipartCXEncoder2, self).__init__(fields=fields)
        self.nice_cx = nice_cx

    def read(self, size=-1):
        """Read data from the streaming encoder.

        :param int size: (optional), If provided, ``read`` will return exactly
            that many bytes. If it is not provided, it will return the
            remaining bytes.
        :returns: bytes
        """
        if self.finished:
            return self._buffer.read(size)

        bytes_to_load = size
        if bytes_to_load != -1 and bytes_to_load is not None:
            bytes_to_load = self._calculate_load_amount(int(size))

        self._load(bytes_to_load)
        return self._buffer.read(size)

class MultipartCXEncoder(object):
    def __init__(self, nice_cx):
        self.boundary = uuid.uuid4().hex
        self.content_type = 'multipart/form-data; boundary={}'.format(self.boundary)
        self.nice_cx = nice_cx

    @classmethod
    def u(cls, s):
        if sys.hexversion < 0x03000000 and isinstance(s, str):
            s = s.decode('utf-8')
        if sys.hexversion >= 0x03000000 and isinstance(s, bytes):
            s = s.decode('utf-8')
        return s

    def iter(self, fields, files):
        """
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, file-type) elements for data to be uploaded as files
        Yield body's chunk as bytes
        """

        encoder = codecs.getencoder('utf-8')
        yield encoder('--{}\r\n'.format(self.boundary))
        yield encoder(self.u('Content-Disposition: form-data; name="{}"\r\n').format('CXNetworkStream'))
        yield encoder('\r\n')
        #if isinstance(value, int) or isinstance(value, float):
        #    value = str(value)
        #yield encoder(self.u(value))

        if self.nice_cx.node_int_id_generator:
            self.nice_cx.node_id_lookup = list(self.nice_cx.node_int_id_generator)

        if self.nice_cx.metadata:
            yield encoder(self.u(json.dumps(self.nice_cx.generateMetadataAspect())))
        '''
        if self.nice_cx.nodes:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('nodes'))))
        if self.nice_cx.edges:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('edges'))))
        if self.nice_cx.networkAttributes:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('networkAttributes'))))
        if self.nice_cx.nodeAttributes:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('nodeAttributes'))))
        if self.nice_cx.edgeAttributes:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('edgeAttributes'))))
        if self.nice_cx.citations:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('citations'))))
        if self.nice_cx.nodeCitations:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('nodeCitations'))))
        if self.nice_cx.edgeCitations:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('edgeCitations'))))
        if self.nice_cx.edgeSupports:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('edgeSupports'))))
        if self.nice_cx.nodeSupports:
            yield encoder(self.u(json.dumps(self.nice_cx.generateAspect('nodeSupports'))))
        if self.nice_cx.opaqueAspects:
            for oa in self.nice_cx.opaqueAspects:
                yield encoder(self.u(json.dumps({oa: self.nice_cx.opaqueAspects[oa]})))
        if self.nice_cx.metadata:
            #===========================
            # UPDATE CONSISTENCY GROUP
            #===========================
            yield encoder(self.u(json.dumps(self.nice_cx.generateMetadataAspect())))
        '''

        yield encoder('\r\n')
        '''
        for (key, filename, fd) in files:
            key = self.u(key)
            filename = self.u(filename)
            yield encoder('--{}\r\n'.format(self.boundary))
            yield encoder(self.u('Content-Disposition: form-data; name="{}"; filename="{}"\r\n').format(key, filename))
            yield encoder('Content-Type: {}\r\n'.format('application/octet-stream'))#'application/json'))
            yield encoder('\r\n')
            with fd:
                buff = fd.read()
                yield (buff, len(buff))
            yield encoder('\r\n')
        '''
        yield encoder('--{}--\r\n'.format(self.boundary))

    def read(self, size=-1):
        """Read data from the streaming encoder.

        :param int size: (optional), If provided, ``read`` will return exactly
            that many bytes. If it is not provided, it will return the
            remaining bytes.
        :returns: bytes
        """
        if self.finished:
            return self._buffer.read(size)

        bytes_to_load = size
        if bytes_to_load != -1 and bytes_to_load is not None:
            bytes_to_load = self._calculate_load_amount(int(size))

        self._load(bytes_to_load)
        return self._buffer.read(size)

    def encode(self, fields, files):
        body = io.BytesIO()
        for chunk, chunk_len in self.iter(fields, files):
            body.write(chunk)
        return self.content_type, body.getvalue()

def postNiceCxStream(niceCx):
    url = 'http://dev2.ndexbio.org/v2/network'

    stream = io.BytesIO(json.dumps({'placeholder':''}))
    fields = {
        'CXNetworkStream': ('filename', stream, 'application/octet-stream')
    }

    multipart_data = MultipartCXEncoder2(fields, niceCx)

    headers = {
        'Content-Type': multipart_data.content_type,
        'User-Agent': 'NDEx-Python/3.0'
    }
    response = requests.post(url, data=multipart_data, headers=headers, auth=('scratch', 'scratch')) # .encode(None, None)




__author__ = 'aarongary'

import unittest

import sys
import time
import ijson
import pickle
import json
import pandas as pd
import csv
import networkx as nx
from nicecxModel.NiceCXNetwork import NiceCXNetwork
from nicecxModel.cx.aspects.NodesElement import NodesElement
from nicecxModel.cx.aspects.EdgesElement import EdgesElement
from nicecxModel.cx.aspects.NetworkAttributesElement import NetworkAttributesElement
from nicecxModel.cx.aspects.NodeAttributesElement import NodeAttributesElement
from nicecxModel.cx.aspects.EdgeAttributesElement import EdgeAttributesElement
from nicecxModel.cx.aspects.CitationElement import CitationElement
from nicecxModel.cx.aspects.SupportElement import SupportElement
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE
from nicecxModel.cx.aspects.SimpleNode import SimpleNode
from nicecxModel.cx import CX_CONSTANTS
from ndex2.NetworkQuery import NetworkQuery
from ndex2.client import Ndex2
from ndex2.NiceCXBuilder import NiceCXBuilder

upload_server = 'dev.ndexbio.org'
upload_username = 'scratch'
upload_password = 'scratch'

class TestLoadByAspects(unittest.TestCase):
    @unittest.skip("Temporary skipping")
    def test_load_nodes(self):
        niceCx = NiceCXNetwork()
        gene_list = ['OR2J3', 'AANAT', 'CCDC158', 'PLAC8L1', 'CLK1', 'GLTP', 'PITPNM2','TRAPPC8', 'EIF2S2', 'ST14',
                     'NXF1', 'H3F3B','FOSB', 'MTMR4', 'USP46', 'CDH11', 'ENAH', 'CNOT7', 'STK39', 'CAPZA1', 'STIM2',
                     'DLL4', 'WEE1', 'MYO1D', 'TEAD3']
        for i in range(1,10):
            niceCx.addNode(NodesElement(id=i, node_name='node%s' % str(i), node_represents=gene_list[i]))

        upload_message = niceCx.upload_to('dev2.ndexbio.org', 'scratch', 'scratch')
        self.assertTrue(upload_message)
        print niceCx.to_json()

    @unittest.skip("Temporary skipping")
    def test_load_edges(self):
        niceCx = NiceCXNetwork()
        niceCx.addNode(NodesElement(id=1, node_name='node%s' % str(1), node_represents='ABC'))
        niceCx.addNode(NodesElement(id=2, node_name='node%s' % str(2), node_represents='DEF'))
        niceCx.addEdge(EdgesElement(id=1, edge_source=1, edge_target=2, edge_interaction='neighbor'))

        upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
        self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_pandas_loading(self):
        with open('MDA1.txt', 'rU') as tsvfile:
            header = [h.strip() for h in tsvfile.next().split('\t')]

            df = pd.read_csv(tsvfile,delimiter='\t',engine='python',names=header)

            niceCx = NiceCXNetwork()
            for index, row in df.iterrows():
                niceCx.addNode(NodesElement(id=row['Bait'], node_name=row['Bait'], node_represents=row['Bait']))
                niceCx.addNode(NodesElement(id=row['Prey'], node_name=row['Prey'], node_represents=row['Prey']))

                niceCx.addEdge(EdgesElement(id=index, edge_source=row['Bait'], edge_target=row['Prey'], edge_interaction='interacts-with'))

            niceCx.add_metadata_stub('nodes')
            niceCx.add_metadata_stub('edges')
            if niceCx.nodeAttributes:
                niceCx.add_metadata_stub('nodeAttributes')
            if niceCx.edgeAttributes:
                niceCx.add_metadata_stub('edgeAttributes')
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)

            print df

        my_df = pd.DataFrame(data=[(4,14),(5,15),(6,16),(7,17)], index=range(0,4), columns=['A','B'])
        self.assertIsNotNone(my_df)
        #print(pd.DataFrame(my_df))

    @unittest.skip("Temporary skipping")
    def test_create_from_pandas_no_headers(self):
        with open('SIMPLE.txt', 'rU') as tsvfile:
            df = pd.read_csv(tsvfile,delimiter='\t',engine='python',header=None)

            niceCx = NiceCXNetwork(pandas_df=df)
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_pandas_with_headers(self):
        with open('MDA1.txt', 'rU') as tsvfile:
            header = [h.strip() for h in tsvfile.next().split('\t')]

            df = pd.read_csv(tsvfile,delimiter='\t',engine='python',names=header)

            niceCx = NiceCXNetwork()
            niceCx.create_from_pandas(df, source_field='Bait', target_field='Prey', source_node_attr=['AvePSM'], target_node_attr=['WD'], edge_attr=['Z', 'Entropy'])
            my_cx_json = niceCx.to_json()
            print json.dumps(my_cx_json)
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_server(self):
        niceCx = NiceCXNetwork(server='dev2.ndexbio.org', username='scratch', password='scratch', uuid='9433a84d-6196-11e5-8ac5-06603eb7f303')
        upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
        self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_pandas_no_headers_3_columns(self):
        with open('SIMPLE3.txt', 'rU') as tsvfile:
            df = pd.read_csv(tsvfile,delimiter='\t',engine='python',header=None)

            #====================================
            # BUILD NICECX FROM PANDAS DATAFRAME
            #====================================
            niceCx = NiceCXNetwork(pandas_df=df)
            niceCx.apply_template('dev2.ndexbio.org', 'scratch', 'scratch', '3daff7cd-9a6b-11e7-9743-0660b7976219')
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_networkx(self):
        with open('SIMPLE3.txt', 'rU') as tsvfile:
            reader = csv.DictReader(filter(lambda row: row[0] != '#', tsvfile), dialect='excel-tab', fieldnames=['s','t','e'])

            #===========================
            # BUILD NETWORKX GRAPH
            #===========================
            G = nx.Graph()
            for row in reader:
                G.add_node(row.get('s'), test1='test1_s', test2='test2_s')
                G.add_node(row.get('t'), test1='test1_t', test2='test2_t')
                G.add_edge(row.get('s'), row.get('t'), {'interaction': 'controls-production-of', 'test3': 'test3'})

            #====================================
            # BUILD NICECX FROM NETWORKX GRAPH
            #====================================
            niceCx = NiceCXNetwork(networkx_G=G)
            niceCx.apply_template('dev2.ndexbio.org', 'scratch', 'scratch', '3daff7cd-9a6b-11e7-9743-0660b7976219')
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_pandas_with_headers2(self):
        with open('CTD_genes_pathways.txt', 'rU') as tsvfile:
            header = [h.strip() for h in tsvfile.next().split('\t')]

            df = pd.read_csv(tsvfile,delimiter='\t',engine='python',names=header)

            niceCx = NiceCXNetwork()
            niceCx.create_from_pandas(df, source_field='GeneSymbol', target_field='PathwayName', source_node_attr=['GeneID'], target_node_attr=['Pathway Source'], edge_attr=[])
            my_cx_json = niceCx.to_json()
            print json.dumps(my_cx_json)
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_cx_file(self):
        with open('MEDIUM_NETWORK.cx', 'rU') as ras_cx:
            #====================================
            # BUILD NICECX FROM PANDAS DATAFRAME
            #====================================
            niceCx = NiceCXNetwork(cx=json.load(ras_cx))
            my_cx = niceCx.to_json()
            print(my_cx)
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_create_from_server_1(self):
        #====================================
        # BUILD NICECX FROM PANDAS DATAFRAME
        #====================================
        niceCx = NiceCXNetwork(server='dev.ndexbio.org', username='scratch', password='scratch', uuid='b7190ca4-aec2-11e7-9b0a-06832d634f41')
        upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
        self.assertTrue(upload_message)

    @unittest.skip("Temporary skipping")
    def test_export_to_cx_file(self):
        with open('MEDIUM_NETWORK.cx', 'rU') as ras_cx:
            #====================================
            # BUILD NICECX FROM PANDAS DATAFRAME
            #====================================
            niceCx = NiceCXNetwork(cx=json.load(ras_cx))
            nice_networkx = niceCx.to_networkx()
            #my_cx = niceCx.to_json()
            print(nice_networkx)
            upload_message = niceCx.upload_to(upload_server, upload_username, upload_password)
            self.assertTrue(upload_message)

    #@unittest.skip("Temporary skipping")
    def test_manual_build(self):
        niceCx = NiceCXNetwork()
        add_this_node_fox = NodesElement(node_name='Fox')
        niceCx.addNode(add_this_node_fox)

        add_this_node_mouse = NodesElement(node_name='Mouse')
        niceCx.addNode(add_this_node_mouse)

        add_this_node_bird = NodesElement(node_name='Bird')
        niceCx.addNode(add_this_node_bird)

        fox_bird_edge = EdgesElement(edge_source=add_this_node_fox.getId(), edge_target=add_this_node_bird.getId(), edge_interaction='interacts-with')
        niceCx.addEdge(fox_bird_edge)

        fox_mouse_edge = EdgesElement(edge_source=add_this_node_fox.getId(), edge_target=add_this_node_mouse.getId(), edge_interaction='interacts-with')
        niceCx.addEdge(fox_mouse_edge)

        niceCx.addNodeAttribute(NodeAttributesElement(property_of=add_this_node_fox.getId(), name='letter_count', values=len(add_this_node_fox.getName())))

        print niceCx



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

if __name__ == '__main__':
    setup(
        name='ndex2',

        # Versions should comply with PEP440.  For a discussion on single-sourcing
        # the version across setup.py and the project code, see
        # https://packaging.python.org/en/latest/single_source_version.html
        version='0.0.1b.8',

        description='Nice CX Python includes a client and a data model.',
        long_description=long_description,

        # The project's main homepage.
        url='https://www.ndexbio.org',

        # Author details
        author='The NDEx Project',
        author_email='contact@ndexbio.org',

        # Choose your license
        license='BSD',

        # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            # How mature is this project? Common values are
            #   3 - Alpha
            #   4 - Beta
            #   5 - Production/Stable
            'Development Status :: 4 - Beta',

            # Indicate who your project is intended for
            'Intended Audience :: Science/Research',
            'Topic :: Scientific/Engineering :: Information Analysis',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
            'Topic :: Scientific/Engineering :: Medical Science Apps.',

            # Pick your license as you wish (should match "license" above)
            'License :: OSI Approved :: BSD License',

            # Specify the Python versions you support here. In particular, ensure
            # that you indicate whether you support Python 2, Python 3 or both.
            'Programming Language :: Python :: 2.7',
        ],

        # What does your project relate to?
        keywords='network analysis biology',

        # You can just specify the packages manually here if your project is
        # simple. Or you can use find_packages().
        packages=find_packages(exclude=[]),

        install_requires = [
            'ijson',
            'enum',
            'requests',
            'requests_toolbelt',
            'networkx==1.11',
            'urllib3>=1.16',
            'pandas',
            'networkx'
        ],

        include_package_data=True
    )



from ndex2.networkn import NdexGraph, FilterSub
import ndex2

from ndex2.client import Ndex2
import networkx as nx
import os
import json
import inspect
import io


def test_set_edge_attribute():
    G = NdexGraph()

    G = NdexGraph()
    G.add_new_node('1')
    G.add_new_node('2')
    G.add_new_node('3')

    edge1 = G.add_edge_between(1, 3)
    edge2 = G.add_edge_between(2, 3)

    G.set_edge_attribute(edge1, 'w', 1.2399)
    G.set_edge_attribute(edge2, 'w', -0.5334)

    #network_id = G.upload_to("http://dev.ndexbio.org", "scratch", "scratch")
    #print(network_id)

    print G.to_cx()

def test_create_from_edge_list():
    G = NdexGraph()
    edge_list = [('A', 'B'), ('B', 'C')]

    G.create_from_edge_list(edge_list, interaction=['A-B', 'B-C'])

    G.set_name('create_from_edge_list')

    G.set_edge_attribute(1, 'w', 1.2399)

    network_id = G.upload_to("http://dev.ndexbio.org", "scratch", "scratch")
    print(network_id)

    print G.to_cx()

    #ndex = Ndex("http://dev.ndexbio.org", "scratch", "scratch")
    #ndex.make_network_public(network_id)
    #ndex.make_network_private(network_id)

#def test_cartesian():
 #   G = NdexGraph(server="http://test.ndexbio.org",
  #                username='scratch', password='scratch',
   #               uuid='aa6e7426-3f14-11e6-a7fa-028f28cd6a5b')
    #G.write_to('cartesian2.cx')

def test_layout():
    nx_G = nx.complete_graph(11)
    # nx_G.remove_node(0)
    # nx_G.remove_node(1)
    G = NdexGraph()
    G.add_new_node('1')
    G.add_new_node('2')
    G.add_new_node('3')
    G.add_new_node('4')
    G.add_new_node('5')
    G.add_new_node('6')

    G.add_edge_between(1, 3)
    G.add_edge_between(1, 4)
    G.add_edge_between(2, 3)
    G.add_edge_between(2, 4)

    G.add_edge_between(3, 5)
    G.add_edge_between(3, 6)
    G.add_edge_between(4, 5)
    G.add_edge_between(4, 6, interaction='AAABBBCCC') #attr_dict={'interaction':'testing'})

    initial_pos = {
        1: (0.0, 1.0),
        2: (0.0, 0.0),
        # 3: (0.5, 0.5),
        # 4: (0.5, 0.5),
        5: (1.0, 1.0),
        6: (1.0, 0.0),
    }

    fixed = [1,2,5,6]

    # G.add_new_node('3')
    # G.add_new_node('4')
    # G.add_new_node('5')
    G.pos = nx.spring_layout(G.to_undirected(), pos=initial_pos, fixed=fixed)
    print(G.pos)
    G.set_name('spring_layout undirected')
    #G.upload_to('http://dev.ndexbio.org','scratch','scratch')


def test_filter_sub():
    repo_directory = '/Users/aarongary/Development/DataSets/NDEx/server2/data/'
    print inspect.getfile(FilterSub)
    read_this_aspect = os.path.join(repo_directory, 'NCI_Style.cx') #'Diffusion1.cx') #'subnetwork_ex1.cx')

    with open(read_this_aspect, 'rt') as fid:
        data = json.load(fid)
        if(data is not None):
            my_filter_sub = FilterSub(data, subnet_index=0)
            ndexGraph = NdexGraph(my_filter_sub.get_cx())
            print ndexGraph.to_cx()


def scratch_test():
    G = NdexGraph()
    G.add_new_node('1')
    G.add_new_node('2')
    G.add_edge_between(1,2)

    G.set_name('scratch-test1 - jing')
    print(G.graph())
    #G.upload_to('http://dev.ndexbio.org', 'scratch', 'scratch')

def test_data_to_type():
    NdexGraph.data_to_type('true','boolean')

def test_cx_string_building():
    cx_str = '[{"numberVerification": [{"longNumber": 281474976710655}]}, {"metaData": [{"idCounter": 4, "name": "nodes"}, {"idCounter": 4, "name": "edges"}]}, {"networkAttributes": [{"v": "indra_assembled", "n": "name"}, {"v": "", "n": "description"}]}, {"nodeAttributes": [{"v": "proteinfamily", "po": 0, "n": "type"}, {"v": "MEK", "po": 0, "n": "BE"}, {"v": "proteinfamily", "po": 1, "n": "type"}, {"v": "C26360", "po": 1, "n": "NCIT"}, {"v": "ERK", "po": 1, "n": "BE"}]}, {"edgeAttributes": [{"v": "Phosphorylation(MEK(), ERK())", "po": 2, "n": "INDRA statement"}, {"v": "Modification", "po": 2, "n": "type"}, {"v": "positive", "po": 2, "n": "polarity"}, {"v": "1.00", "po": 2, "n": "Belief score"}, {"v": "MEK phosphorylates ERK.", "po": 2, "n": "Text"}]}, {"edges": [{"i": "Phosphorylation", "s": 0, "@id": 2, "t": 1}]}, {"edgeSupports": [{"supports": [3], "po": [2]}]}, {"citations": []}, {"nodes": [{"@id": 0, "n": "MEK"}, {"@id": 1, "n": "ERK"}]}, {"supports": [{"text": "MEK phosphorylates ERK.", "@id": 3}]}, {"edgeCitations": []},{"status": [{"error": "","success": true}]}]'
    stream = io.BytesIO(cx_str)
    nd = ndex2.client.Ndex2('http://preview.ndexbio.org',
                                 username='scratch',
                                 password='scratch')

    network_id = nd.save_cx_stream_as_new_network(stream)

if __name__ == "__main__":
    #test_layout()
    test_set_edge_attribute()




__author__ = 'aarongary'

from nicecxModel.cx import CX_CONSTANTS
import json

class CitationElement(object):
    def __init__(self, id=None, title=None, contributor=None, identifier=None, type=None, description=None, attributes=None, json_obj=None):
        self.ASPECT_NAME = 'citations'

        if json_obj is None:
            self._id = id
            self._title = title
            self._contributor = contributor
            self._identifier = identifier
            self._type = type
            self._description = description
            self._attributes = attributes
        else:
            self._id = json_obj.get(CX_CONSTANTS.ID)
            self._title = json_obj.get(CX_CONSTANTS.CITATION_TITLE)
            self._contributor = json_obj.get(CX_CONSTANTS.CITATION_CONTRIBUTOR)
            self._identifier = json_obj.get(CX_CONSTANTS.CITATION_IDENTIFIER)
            self._type = json_obj.get(CX_CONSTANTS.CITATION_TYPE)
            self._description = json_obj.get(CX_CONSTANTS.CITATION_DESCRIPTION)
            self._attributes = json_obj.get(CX_CONSTANTS.ATTRIBUTES)

    def getId(self):
        return self._id

    def getTile(self):
        return self._title

    def getContributor(self):
        return self._title

    def getIdentifier(self):
        return self._title

    def getType(self):
        return self._title

    def getDescription(self):
        return self._title

    def getAttributes(self):
        return self._title

    def setTile(self, title):
        self._title = title

    def setContributor(self, contributor):
        self._contributor = contributor

    def setIdentifier(self, identifier):
        self._identifier = identifier

    def setType(self, type):
        self._type = type

    def setDescription(self, description):
        self._description = description

    def setAttributes(self, attributes):
        self._attributes = attributes

    def __str__(self):
        return json.dumps(self.to_json())

    def to_json(self):
        return_json = {}

        if self._id == -1:
            raise Exception('Edge element does not have a valid ID.  Unable to process this edge - ' + self._node_name)

        return_json[CX_CONSTANTS.ID] = self._id

        if self._contributor is not None:
            return_json[CX_CONSTANTS.CITATION_CONTRIBUTOR] = self._contributor

        if self._identifier is not None:
            return_json[CX_CONSTANTS.CITATION_IDENTIFIER] = self._identifier

        if self._type is not None:
            return_json[CX_CONSTANTS.CITATION_TYPE] = self._type

        if self._title is not None:
            return_json[CX_CONSTANTS.CITATION_TITLE] = self._title

        if self._description is not None:
            return_json[CX_CONSTANTS.CITATION_DESCRIPTION] = self._description

        if self._attributes is not None:
            return_json[CX_CONSTANTS.ATTRIBUTES] = self._attributes

        return return_json




__author__ = 'aarongary'

import json
from nicecxModel.cx.aspects.AttributeCommon import AttributeCommon
from nicecxModel.cx.aspects.DataModelsUtil import DatamodelsUtil
from nicecxModel.cx.aspects import ATTRIBUTE_DATA_TYPE

class NetworkAttributesElement(AttributeCommon):
    def __init__(self, subnetwork=None, property_of=None, name=None, values=None, type=None, json_obj=None):
        super(NetworkAttributesElement, self).__init__(subnetwork=subnetwork, property_of=property_of, name=name, values=values, type=type, json_obj=json_obj)
        self.ASPECT_NAME = 'networkAttributes'

    def __str__(self):
        return json.dumps(self.to_json())

    def createInstanceWithSingleValue(self, subnetwork, name, value, type):
        return NetworkAttributesElement(subnetwork, name, DatamodelsUtil.removeParenthesis(value, type), type)

    def createInstanceWithMultipleValues(self, subnetwork, name, values, type):
        return NetworkAttributesElement(subnetwork, name, DatamodelsUtil.parseStringToStringList(values, type), type)

    def createInstanceWithJsonValue(self, subnetwork, name, serializedValue, type):
        if ATTRIBUTE_DATA_TYPE.isSingleValueType(type):
            return NetworkAttributesElement(subnetwork, name, serializedValue, type)
        else:
            try:
                sl = json.loads(serializedValue)

                return NetworkAttributesElement(subnetwork, name, sl, type)
            except Exception as e:
                raise Exception('Failed to load json string to json object %s %s %s' % (str(subnetwork), str(name), str(sl)))





import unittest

import ndex2.client as nc

ndex_host = "http://dev2.ndexbio.org"

#
# Python Client APIs tested:
#
#   get_user_by_username
#

class MyTestCase(unittest.TestCase):

    def test_get_user_by_username(self):

        ndex = nc.Ndex2(host=ndex_host)

        user_ttt = ndex.get_user_by_username('scratch')
        self.assertTrue(str(user_ttt['externalId']) == '6b8b841c-43d1-11e6-a5c7-06603eb7f303')
        self.assertTrue(str(user_ttt['firstName']) == 'scratch')
        self.assertTrue(str(user_ttt['lastName']) == 'scratch')
        self.assertTrue(str(user_ttt['userName']) == 'scratch')
        self.assertTrue(str(user_ttt['emailAddress']) == 'scratch@example.com')

        user_drh = ndex.get_user_by_username('drh')
        self.assertTrue(str(user_drh['externalId']) == '972e6b69-b3bd-11e4-ae6e-000c29cb28fb')
        self.assertTrue(str(user_drh['firstName']) == 'Dexter')
        self.assertTrue(str(user_drh['lastName']) == 'Pratt')
        self.assertTrue(str(user_drh['userName']) == 'drh')
        self.assertTrue(str(user_drh['emailAddress']) == 'drh@example.com')

if __name__ == '__main__':
    unittest.main()




__author__ = 'aarongary'

from enum import Enum

known_aspects = [
    'nodes',
    'edges',
    'nodeAttributes',
    'edgeAttributes',
    'networkAttributes',
    'provenanceHistory',
    'citations',
    'nodeCitations',
    'edgeCitations',
    'supports',
    'nodeSupports',
    'edgeSupports',
    'cartesianLayout',
    '@context',
    'cyVisualProperties',
    'visualProperties'
    ]

known_aspects_min = [
    'nodes',
    'edges',
    'nodeAttributes',
    'edgeAttributes',
    'networkAttributes',
    'provenanceHistory',
    'citations',
    'nodeCitations',
    'edgeCitations',
    'supports',
    'nodeSupports',
    'edgeSupports'
    ]

class CX_CONSTANTS(str, Enum):
    ID = '@id'
    NAME = 'n'
    VALUE = 'v'
    NODE_REPRESENTS = 'r'
    EDGE_INTERACTION = 'i'
    EDGE_SOURCE_NODE_ID_OR_SUBNETWORK = 's'
    EDGE_TARGET_NODE_ID = 't'
    PROPERTY_OF = 'po'
    DATA_TYPE = 'd'
    CITATION_TITLE = "dc:title"
    CITATION_CONTRIBUTOR = "dc:contributor"
    CITATION_IDENTIFIER = "dc:identifier"
    CITATION_TYPE = "dc:type"
    CITATION_DESCRIPTION = "dc:description"
    ATTRIBUTES = 'attributes'
    TEXT = 'text'
    CITATION = 'citation'
    CITATIONS = 'citations'
    SUPPORTS = 'supports'
    PROPERTIES = 'properties'

    #======================
    # METADATA CONSTANTS
    #======================
    CONSISTENCY_GROUP = 'consistencyGroup'
    ELEMENT_COUNT = 'elementCount'
    ID_COUNTER = 'idCounter'
    LAST_UPDATE = 'lastUpdate'
    METADATA_NAME = 'name'
    VERSION = 'version'
